'''
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re

re_word = re.compile(r'^[\w\-:\$]+')
re_attr_string = re.compile(r'^(["\'])((?:(?!\1)[^\\]|\\.)*)\1')
re_valid_name = re.compile(r'^[\w\d\-_\$\:@!]+\+?$', re.IGNORECASE)

def char_at(text, pos):
	"""
	Returns character at specified index of text.
	If index if out of range, returns empty string
	"""
	return text[pos] if pos < len(text) else ''

def split_expression(expr):
	"""
	Split expression by node name and its content, if exists. E.g. if we pass
	'a{Text}' expression, it will be splitted into 'a' and 'Text'
	
	@type expr: str
	@return: Result tuple with two elements: node name and its content
	"""
	# fast test on text node
	if '{' not in expr:
		return expr, None
		
	attr_lvl = 0
	text_lvl = 0
	brace_stack = []
	i = 0
	il = len(expr)
		
	while i < il:
		ch = expr[i]
		if ch == '[':
			if not text_lvl:
				attr_lvl += 1
		elif ch == ']':
			if not text_lvl:
				attr_lvl -= 1
		elif ch == '{':
			if not attr_lvl:
				text_lvl += 1
				brace_stack.append(i)
		elif ch == '}':
			if not attr_lvl:
				text_lvl -= 1
				brace_start = brace_stack.pop()
				if text_lvl == 0:
					# found braces bounds
					return expr[0:brace_start], expr[brace_start + 1:i]
		i += 1
	
	# if we are here, then no valid text node found
	return expr, None

def parse_attributes(s):
	"""
	Parses tag attributes extracted from abbreviation
	@type s: str
	
	Example of incoming data:
	#header
	.some.data
	.some.data#header
	[attr]
	#item[attr=Hello other="World"].class
	"""
	
	result = []
	name = ''
	collect_name = True
	class_name = None
	char_map = {'#': 'id', '.': 'class'}
	
	# walk char-by-char
	i = 0
	il = len(s)
		
	while i < il:
		ch = s[i]
		if ch == '#': # id
			val = get_word(i, s[1:])
			result.append({'name': char_map[ch], 'value': val})
			i += len(val) + 1
			collect_name = False
		elif ch == '.': # class
			val = get_word(i, s[1:])
			if not class_name:
				# remember object pointer for value modification
				class_name = {'name': char_map[ch], 'value': ''}
				result.append(class_name)
			
			class_name['value'] += (class_name['value'] and ' ' or '') + val
			i += len(val) + 1
			collect_name = False
		elif ch == '[': # begin attribute set
			# search for end of set
			try:
				end_ix = s.index(']', i)
				for a in extract_attributes(s[i + 1:end_ix]):
					result.append(a)
					
				i = end_ix
			except:
				# invalid attribute set, stop searching
				i = len(s)
				
			collect_name = False
		else:
			if collect_name:
				name += ch
			i += 1
			
	return name, result

def get_word(ix, s):
	"""
	Get word, starting at 'ix' character of 's
	"""
	m = re_word.match(s[ix:])
	return m and m.group(0) or ''

def extract_attributes(attr_set):
	"""
	Extract attributes and their values from attribute set 
	@type attr_set: str
	"""
	attr_set = attr_set.strip()
	loop_count = 100 # endless loop protection
	result = []
	attr = None
		
	while attr_set and loop_count:
		attr_name = get_word(0, attr_set)
		attr = None
		if attr_name:
			attr = {'name': attr_name, 'value': ''}
			
			# let's see if attribute has value
			ch = char_at(attr_set, len(attr_name))
			if ch == '=':
				ch2 = char_at(attr_set, len(attr_name) + 1)
				if ch2 == '"' or ch2 == "'":
					# we have a quoted string
					m = re_attr_string.match(attr_set[len(attr_name) + 1:])
					if m:
						attr['value'] = m.group(2)
						attr_set = attr_set[len(attr_name) + len(m.group(0)) + 1:].strip()
					else:
						# something wrong, break loop
						attr_set = ''
				else:
					# unquoted string
					m = re.match(r'(.+?)(\s|$)', attr_set[len(attr_name) + 1:])
					if m:
						attr['value'] = m.group(1)
						attr_set = attr_set[len(attr_name) + len(m.group(1)) + 1:].strip()
					else:
						# something wrong, break loop
						attr_set = ''
			else:
				attr_set = attr_set[len(attr_name):].strip()
		else:
			# something wrong, can't extract attribute name
			break;
		
		if attr: result.append(attr)
		loop_count -= 1
	
	return result

def squash(node):
	"""
	Optimizes tree node: replaces empty nodes with their children
	@type node: TreeNode
	@return: TreeNode
	"""
	for i, child in enumerate(node.children):
		if child.is_empty():
			node.children[i:i + 1] = child.children
	
	return node

def optimize_tree(node):
	"""
	@type node: TreeNode
	@return: TreeNode
	"""
	while node.has_empty_children():
		squash(node)
		
	for child in node.children:
		optimize_tree(child)
	
	return node

def parse(abbr):
	"""
	Parses abbreviation into tree with respect of groups, 
	text nodes and attributes. Each node of the tree is a single 
	abbreviation. Tree represents actual structure of the outputted 
	result
	@param abbr: Abbreviation to parse
	@type abbr: str
	@return: TreeNode
	"""
	root = TreeNode()
	context = root.add_child()
	i = 0
	il = len(abbr)
	text_lvl = 0
	attr_lvl = 0
	group_stack = [root]
	token = ['']
		
	def dump_token():
		if token[0]:
			context.set_abbreviation(token[0])
		token[0] = ''
		
	while i < il:
		ch = abbr[i]
		prev_ch = i and abbr[i - 1] or ''
		if ch == '{':
			if not attr_lvl:
				text_lvl += 1
			token[0] += ch
		elif ch == '}':
			if not attr_lvl:
				text_lvl -= 1
			token[0] += ch
		elif ch == '[':
			if not text_lvl:
				attr_lvl += 1
			token[0] += ch
		elif ch == ']':
			if not text_lvl:
				attr_lvl -= 1
			token[0] += ch
		elif ch == '(':
			if not text_lvl and not attr_lvl:
				# beginning of the new group
				dump_token();
				
				if prev_ch != '+' and prev_ch != '>':
					# previous char is not an operator, assume it's
					# a sibling
					context = context.parent.add_child()
				
				group_stack.append(context)
				context = context.add_child()
			else:
				token[0] += ch
		elif ch == ')':
			if not text_lvl and not attr_lvl:
				# end of the group, pop stack
				dump_token()
				context = group_stack.pop()
				
				if i < il - 1 and char_at(abbr, i + 1) == '*':
					# group multiplication
					group_mul = ''
					for j in xrange(i + 2, il):
						n_ch = abbr[j]
						if n_ch.isdigit():
							group_mul += n_ch
						else:
							break
						
					i += len(group_mul) + 1
					group_mul = int(group_mul or 1)
					while 1 < group_mul:
						context.parent.add_child(context)
						group_mul -= 1
			else:
				token[0] += ch
		elif ch == '+': # sibling operator
			if not text_lvl and not attr_lvl and i != il - 1:
				dump_token()
				context = context.parent.add_child()
			else:
				token[0] += ch
		elif ch == '>': # child operator
			if not text_lvl and not attr_lvl:
				dump_token()
				context = context.add_child()
			else:
				token[0] += ch
		else:
			token[0] += ch
		
		i += 1
	
	# put the final token
	dump_token()
	return optimize_tree(root)

class TreeNode(object):
	re_multiplier = re.compile(r'\*(\d+)?$')
	
	def __init__(self, parent=None):
		self.abbreviation = '';
		self.parent = parent
		self.children = []
		self.count = 1
		self.name = None
		self.text = None
		self.attributes = []
		self.is_repeating = False
		self.has_implicit_name = False
		
	def add_child(self, child=None):
		"""
		Adds passed or creates new child
		@type child: TreeNode
		@return: TreeNode
		"""
		if not child: child = TreeNode()
		child.parent = self
		self.children.append(child)
		return child
	
	def replace(self, node):
		"""
		Replace current node in parent's child list with another node
		@type node: TreeNode
		"""
		if self.parent:
			children = self.parent.children
			if self in children:
				children[children.index(self)] = node
				self.parent = None
				return
			
	def set_abbreviation(self, abbr):
		"""
		Sets abbreviation that belongs to current node
		@type abbr: str
		"""
		self.abbreviation = abbr
		m = self.re_multiplier.search(abbr)
		if m:
			self.count = m.group(1) and int(m.group(1)) or 1
			self.is_repeating = not m.group(1)
			abbr = abbr[0:-len(m.group(0))]
		
		if abbr:
			name, self.text = split_expression(abbr)
				
			if name:
				self.name, self.attributes = parse_attributes(name)
				if not self.name:
					self.name = 'div'
					self.has_implicit_name = True
		
		# validate name
		if self.name and not re_valid_name.match(self.name):
			raise ZenInvalidAbbreviation('self.name')
		
	def get_abbreviation(self):
		return self.expr
	
	def to_string(self, level=0):
		"""
		Dump current tree node into a foramtted string
		"""
		output = '(empty)'
		if self.abbreviation:
			output = ''
			if self.name:
				output = self.name
				
			if self.text is not None:
				output += (output and ' ' or '') + '{text: "' + self.text + '"}'
				
			if self.attributes:
				output += ' [' + ', '.join(['%s="%s"' % (a['name'], a['value']) for a in self.attributes]) + ']'
			
		result = ('-' * level) + output + '\n'
		for child in self.children:
			result += child.to_string(level + 1)
			
		return result
	
	def __repr__(self):
		return self.to_string()
	
	def has_empty_children(self):
		"""
		Check if current node contains children with empty <code>expr</code>
		property
		"""
		for child in self.children:
			if child.is_empty():
				return True
			
		return False
	
	def is_empty(self):
		return not self.abbreviation
	
	def is_text_node(self):
		"""
		Check if current node is a text-only node
		"""
		return not self.name and self.text
	

class ZenInvalidAbbreviation(Exception):
	"""
	Invalid abbreviation error
	@since: 0.7
	"""
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Filter that produces HAML tree
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import zencoding.utils

child_token = '${child}'
tabstops = [0]
	
def make_attributes_string(tag, profile):
	"""
	 Creates HTML attributes string from tag according to profile settings
	 @type tag: ZenNode
	 @type profile: dict
	"""
	# make attribute string
	attrs = ''
	attr_quote = profile['attr_quotes'] == 'single' and "'" or '"'
	cursor = profile['place_cursor'] and zencoding.utils.get_caret_placeholder() or ''
		
	# use short notation for ID and CLASS attributes
	for a in tag.attributes:
		name_lower = a['name'].lower()
		if name_lower == 'id':
			attrs += '#' + (a['value'] or cursor)
		elif name_lower == 'class':
			attrs += '.' + (a['value'] or cursor)
			
	other_attrs = []
	
	# process other attributes
	for a in tag.attributes:
		name_lower = a['name'].lower()
		if name_lower != 'id' and name_lower != 'class':
			attr_name = profile['attr_case'] == 'upper' and a['name'].upper() or name_lower
			other_attrs.append(':' + attr_name + ' => ' + attr_quote + (a['value'] or cursor) + attr_quote)
		
	if other_attrs:
		attrs += '{' + ', '.join(other_attrs) + '}'
	
	return attrs

def _replace(placeholder, value):
	if placeholder:
		return placeholder % value
	else:
		return value		

def process_snippet(item, profile, level=0):
	"""
	Processes element with <code>snippet</code> type
	@type item: ZenNode
	@type profile: dict
	@type level: int
	"""
	data = item.source.value
		
	if not data:
		# snippet wasn't found, process it as tag
		return process_tag(item, profile, level)
		
	tokens = data.split(child_token)
	if len(tokens) < 2:
		start = tokens[0]
		end = ''
	else:
		start, end = tokens
	
	padding = item.parent and item.parent.padding or ''
		
	item.start = _replace(item.start, zencoding.utils.pad_string(start, padding))
	item.end = _replace(item.end, zencoding.utils.pad_string(end, padding))
	
	# replace variables ID and CLASS
	def cb(m):
		if m.group(1) == 'id' or m.group(1) == 'class':
			return item.get_attribute(m.group(1))
		else:
			return m.group(0)
	
	item.start = zencoding.utils.replace_variables(item.start, cb)
	item.end = zencoding.utils.replace_variables(item.end, cb)
	
	return item

def has_block_sibling(item):
	"""
	Test if passed node has block-level sibling element
	@type item: ZenNode
	@return: bool
	"""
	return item.parent and item.parent.has_block_children()

def process_tag(item, profile, level=0):
	"""
	Processes element with <code>tag</code> type
	@type item: ZenNode
	@type profile: dict
	@type level: int
	"""
	if not item.name:
		# looks like it's root element
		return item
	
	attrs = make_attributes_string(item, profile) 
	cursor = profile['place_cursor'] and zencoding.utils.get_caret_placeholder() or ''
	self_closing = ''
	is_unary = item.is_unary() and not item.children
	
	if profile['self_closing_tag'] and is_unary:
		self_closing = '/'
		
	# define tag name
	tag_name = '%' + (profile['tag_case'] == 'upper' and item.name.upper() or item.name.lower())
					
	if tag_name.lower() == '%div' and '{' not in attrs:
		# omit div tag
		tag_name = ''
		
	item.end = ''
	item.start = _replace(item.start, tag_name + attrs + self_closing)
	
	if not item.children and not is_unary:
		item.start += cursor
	
	return item

@zencoding.filter('haml')
def process(tree, profile, level=0):
	"""
	Processes simplified tree, making it suitable for output as HTML structure
	@type tree: ZenNode
	@type profile: dict
	@type level: int
	"""
	if level == 0:
		# preformat tree
		tree = zencoding.run_filters(tree, profile, '_format')
		tabstops[0] = 0
		
	for item in tree.children:
		if item.type == 'tag':
			process_tag(item, profile, level)
		else:
			process_snippet(item, profile, level)
	
		# replace counters
		counter = zencoding.utils.get_counter_for_node(item)
		item.start = zencoding.utils.unescape_text(zencoding.utils.replace_counter(item.start, counter))
		item.end = zencoding.utils.unescape_text(zencoding.utils.replace_counter(item.end, counter))
		
		tabstops[0] += zencoding.utils.upgrade_tabstops(item, tabstops[0]) + 1
		
		process(item, profile, level + 1)
		
	return tree


'''
Actions that use stream parsers and tokenizers for traversing:
-- Search for next/previous items in HTML
-- Search for next/previous items in CSS

Created on Jan 21, 2011

@author: sergey
'''
import re
import zencoding
import zencoding.parser.utils as parser_utils

start_tag = re.compile('^<([\w\:\-]+)((?:\s+[\w\-:]+(?:\s*=\s*(?:(?:"[^"]*")|(?:\'[^\']*\')|[^>\s]+))?)*)\s*(\/?)>')
known_xml_types = ['xml-tagname','xml-attname', 'xml-attribute']
known_css_types = ['selector', 'identifier', 'value']

def find_next_html_item(editor):
	"""
	Find next HTML item
	@param editor: ZenEditor
	"""
	is_first = [True]
	
	def fn(content, search_pos, is_backward=False):
		if is_first[0]:
			is_first[0] = False
			return find_opening_tag_from_position(content, search_pos)
		else:
			return get_opening_tag_from_position(content, search_pos)
	
	return find_item(editor, False, fn, get_range_for_next_item_in_html)

def find_prev_html_item(editor):
	"""
	Find previous HTML item
	@param editor: ZenEditor
	"""
	return find_item(editor, True, get_opening_tag_from_position, get_range_for_prev_item_in_html)

def get_range_for_next_item_in_html(tag, offset, sel_start, sel_end):
	"""
	Returns range for item to be selected in tag after current caret position
	@param tag: Tag declaration
	@type tag: str
	
	@param offset: Tag's position index inside content
	@type offset: int
	
	@param sel_start: Start index of user selection
	@type sel_start: int
	
	@param sel_end: End index of user selection
	@type sel_end: int
	
	@return List with two indexes if next item was found, None otherwise
	"""
	tokens = parser_utils.parse_html(tag, offset)
	next = []
			
	# search for token that is right to selection
	for i, token in enumerate(tokens):
		if token['type'] in known_xml_types:
			# check token position
			pos_test = token['start'] >= sel_start
			if token['type'] == 'xml-attribute' and is_quote(token['content'][0]):
				pos_test = token['start'] + 1 >= sel_start and token['end'] -1 != sel_end
			
			if not pos_test and not (sel_start == sel_end and token['end'] > sel_start):
				continue
			
			# found token that should be selected
			if token['type'] == 'xml-attname':
				next = handle_full_attribute_html(tokens, i, sel_end <= token['end'] and token['start'] or -1)
				if next:
					return next
			elif token['end'] > sel_end:
				next = [token['start'], token['end']]
				
				if token['type'] == 'xml-attribute':
					next = handle_quotes_html(token['content'], next)
					
				if sel_start == next[0] and sel_end == next[1]:
					# in case of empty attribute
					continue
				
				return next
		
	return None

def get_range_for_prev_item_in_html(tag, offset, sel_start, sel_end):
	"""
	Returns range for item to be selected in tag before current caret position
	@param tag: Tag declaration
	@type tag: str
	
	@param offset: Tag's position index inside content
	@type offset: int
	
	@param sel_start: Start index of user selection
	@type sel_start: int
	
	@param sel_end: End index of user selection
	@type sel_end: int
	
	@return List with two indexes if next item was found, None otherwise
	"""
	tokens = parser_utils.parse_html(tag, offset)
			
	# search for token that is left to the selection
	for i in range(len(tokens) - 1, -1, -1):
		token = tokens[i]
		if token['type'] in known_xml_types:
			# check token position
			pos_test = token['start'] < sel_start
			if token['type'] == 'xml-attribute' and is_quote(token['content'][0]):
				pos_test = token['start'] + 1 < sel_start
			
			if not pos_test: continue
			
			# found token that should be selected
			if token['type'] == 'xml-attname':
				next = handle_full_attribute_html(tokens, i, token['start'])
				if next: return next
			else:
				next = [token['start'], token['end']]
				
				if token['type'] == 'xml-attribute':
					next = handle_quotes_html(token['content'], next)
				
				return next
		
	return None

def find_opening_tag_from_position(html, pos):
	"""
	Search for opening tag in content, starting at specified position
	@param html: Where to search tag
	@type html: str
	
	@param pos: Character index where to start searching
	@type pos: int
	
	@return: List with tag indexes if valid opening tag was found, None otherwise
	"""
	while pos >= 0:
		tag = get_opening_tag_from_position(html, pos)
		if tag:
			return tag
		pos -= 1
	
	return None

def get_opening_tag_from_position(html, pos, is_backward=False):
	"""
	@param html: Where to search tag
	@type html: str
	
	@param pos: Character index where to start searching
	@type pos: int
	
	@return: List with tag indexes if valid opening tag was found, None otherwise
	"""
	if html[pos] == '<':
		m = re.search(start_tag, html[pos:])
		if m:
			return [pos, pos + len(m.group(0))]
		
	return None


def is_quote(ch):
	return ch == '"' or ch == "'"

def find_item(editor, is_backward, extract_fn, range_fn):
	"""
	@type editor: ZenEditor
	
	@param is_backward: Search backward (search forward otherwise)
	@type is_backward: boolean
	
	@param extract_fn: Function that extracts item content
	@type extract_fn: function
	
	@param range_rn: Function that search for next token range
	@type range_rn: function
	"""
	content = editor.get_content()
	c_len = len(content)
	loop = 100000 # endless loop protection
	prev_range = [-1, -1]
	_sel_start, _sel_end = editor.get_selection_range()
	sel_start = min(_sel_start, _sel_end)
	sel_end = max(_sel_start, _sel_end)
		
	search_pos = sel_start
	while search_pos >= 0 and search_pos < c_len and loop > 0:
		loop -= 1
		item = extract_fn(content, search_pos, is_backward)
		if item:
			if prev_range[0] == item[0] and prev_range[1] == item[1]:
				break
			
			prev_range[0] = item[0]
			prev_range[1] = item[1]
			item_def = content[item[0]:item[1]]
			rng = range_fn(item_def, item[0], sel_start, sel_end)
				
			if rng:
				editor.create_selection(rng[0], rng[1])
				return True
			else:
				search_pos = is_backward and item[0] or item[1] - 1
		
		search_pos += is_backward and -1 or 1
	
	return False

def find_next_css_item(editor):
	return find_item(editor, False, parser_utils.extract_css_rule, get_range_for_next_item_in_css)

def find_prev_css_item(editor):
	return find_item(editor, True, parser_utils.extract_css_rule, get_range_for_prev_item_in_css)


def get_range_for_next_item_in_css(rule, offset, sel_start, sel_end):
	"""
    Returns range for item to be selected in tag after current caret position
    
    @param rule: CSS rule declaration
    @type rule: str
    
    @param offset: Rule's position index inside content
    @type offset: int
    
    @param sel_start: Start index of user selection
    @type sel_start: int
    
    @param sel_end: End index of user selection
    @type sel_end: int
    
    @return: List with two indexes if next item was found, None otherwise
	"""
	tokens = parser_utils.parse_css(rule, offset)
	next = []
		
	def check_same_range(r):
		"Same range is used inside complex value processor"
		return r[0] == sel_start and r[1] == sel_end
	
			
	# search for token that is right to selection
	for i, token in enumerate(tokens):
		if token['type'] in known_css_types:
			# check token position
			if sel_start == sel_end:
				pos_test = token['end'] > sel_start
			else:
				pos_test = token['start'] >= sel_start
				if token['type'] == 'value': # respect complex values
					pos_test = pos_test or sel_start >= token['start'] and token['end'] >= sel_end
			
			if not pos_test: continue
			
			# found token that should be selected
			if token['type'] == 'identifier':
				rule_sel = handle_full_rule_css(tokens, i, sel_end <= token['end'] and token['start'] or -1)
				if rule_sel:
					return rule_sel
				
			elif token['type'] == 'value' and sel_end > token['start'] and token['children']:
				# looks like a complex value
				children = token['children']
				for child in children:
					if child[0] >= sel_start or (sel_start == sel_end and child[1] > sel_start):
						next = [child[0], child[1]]
						if check_same_range(next):
							rule_sel = handle_css_special_case(rule, next[0], next[1], offset)
							if not check_same_range(rule_sel):
								return rule_sel
							else:
								continue
						
						return next
			elif token['end'] > sel_end:
				return [token['start'], token['end']]
	
	return None

def get_range_for_prev_item_in_css(rule, offset, sel_start, sel_end):
	"""
	Returns range for item to be selected in CSS rule before current caret position
	
	@param rule: CSS rule declaration
    @type rule: str
    
    @param offset: Rule's position index inside content
    @type offset: int
    
    @param sel_start: Start index of user selection
    @type sel_start: int
    
    @param sel_end: End index of user selection
    @type sel_end: int
    
    @return: List with two indexes if next item was found, None otherwise
	"""
	tokens = parser_utils.parse_css(rule, offset)
	next = []
			
	def check_same_range(r):
		"Same range is used inside complex value processor"
		return r[0] == sel_start and r[1] == sel_end
		
	# search for token that is left to the selection
	for i in range(len(tokens) - 1, -1, -1):
		token = tokens[i]
		if token['type'] in known_css_types:
			# check token position
			pos_test = token['start'] < sel_start
			if token['type'] == 'value' and token['ref_start_ix'] != token['ref_end_ix']: # respect complex values
				pos_test = token['start'] <= sel_start
			
			if not pos_test: continue
			
			# found token that should be selected
			if token['type'] == 'identifier':
				rule_sel = handle_full_rule_css(tokens, i, token['start'])
				if rule_sel:
					return rule_sel
			elif token['type'] == 'value' and token['ref_start_ix'] != token['ref_end_ix']:
				# looks like a complex value
				children = token['children']
				for child in children:
					if child[0] < sel_start:
						# create array copy
						next = [child[0], child[1]] 
						
						rule_sel = handle_css_special_case(rule, next[0], next[1], offset)
						return not check_same_range(rule_sel) and rule_sel or next
					
				# if we are here than we already traversed trough all
				# child tokens, select full value
				next = [token['start'], token['end']]
				if not check_same_range(next): 
					return next
			else:
				return [token['start'], token['end']]
	
	return None

def handle_full_rule_css(tokens, i, start):
	for t in tokens[i+1:]:
		if (t['type'] == 'value' and start == -1) or t['type'] == 'identifier':
			return [t['start'], t['end']]
		elif t['type'] == ';':
			return [start == -1 and t['start'] or start, t['end']]
		elif t['type'] == '}':
			return [start == -1 and t['start'] or start, t['start'] - 1]
		
	return None

def handle_full_attribute_html(tokens, i, start):
	for t in tokens[i+1:]:
		if t['type'] == 'xml-attribute':
			if start == -1:
				return handle_quotes_html(t['content'], [t['start'], t['end']])
			else:
				return [start, t['end']]
		elif t['type'] == 'xml-attname':
			# moved to next attribute, adjust selection
			return [t['start'], tokens[i]['end']]
		
	return None

def handle_quotes_html(attr, r):
	if is_quote(attr[0]):
		r[0] += 1
	if is_quote(attr[-1]):
		r[1] -= 1
		
	return r

def handle_css_special_case(text, start, end, offset):
	text = text[start - offset:end - offset]
	m = re.match(r'^[\w\-]+\([\'"]?', text)
	if m:
		start += len(m.group(0))
		m = re.search(r'[\'"]?\)$', text)
		if m:
			end -= len(m.group(0))
	
	return [start, end]

@zencoding.action 
def select_next_item(editor):
	if editor.get_syntax() == 'css':
		return find_next_css_item(editor)
	else:
		return find_next_html_item(editor)

@zencoding.action	
def select_previous_item(editor):
	if editor.get_syntax() == 'css':
		return find_prev_css_item(editor)
	else:
		return find_prev_html_item(editor)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Output abbreviation on a single line (i.e. no line breaks)
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re
import zencoding

re_nl = re.compile(r'[\n\r]')
re_pad = re.compile(r'^\s+')

@zencoding.filter('s')
def process(tree, profile):
	for item in tree.children:
		if item.type == 'tag':
			# remove padding from item 
			item.start = re_pad.sub('', item.start)
			item.end = re_pad.sub('', item.end)
		
		# remove newlines 
		item.start = re_nl.sub('', item.start)
		item.end = re_nl.sub('', item.end)
		item.content = re_nl.sub('', item.content)
		
		process(item, profile)
	
	return tree


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Created on Jan 21, 2011

@author: sergey
'''
from zencoding.actions.basic import starts_with
from zencoding.utils import prettify_number
import base64
import math
import re
import zencoding
import zencoding.interface.file as zen_file
import zencoding.parser.utils as parser_utils

@zencoding.action
def reflect_css_value(editor):
	"""
	Reflect CSS value: takes rule's value under caret and pastes it for the same 
	rules with vendor prefixes
	@param editor: ZenEditor
	"""
	if editor.get_syntax() != 'css':
		return False
	
	return compound_update(editor, do_css_reflection(editor))

@zencoding.action
def update_image_size(editor):
	"""
	Update image size: reads image from image/CSS rule under caret
	and updates dimensions inside tag/rule
	@type editor: ZenEditor
	"""
	if editor.get_syntax() == 'css':
		result = update_image_size_css(editor)
	else:
		result = update_image_size_html(editor)
	
	return compound_update(editor, result)

def compound_update(editor, data):
	if data:
		text = data['data']
		
		sel_start, sel_end = editor.get_selection_range()
		
		# try to preserve caret position
		if data['caret'] < data['start'] + len(text):
			relative_pos = data['caret'] - data['start']
			if relative_pos >= 0:
				text = text[:relative_pos] + zencoding.utils.get_caret_placeholder() + text[relative_pos:]
		
		editor.replace_content(text, data['start'], data['end'], True)
#		editor.replace_content(zencoding.utils.unindent(editor, text), data['start'], data['end'])
		editor.create_selection(data['caret'], data['caret'] + sel_end - sel_start)
		return True
	
	return False

def update_image_size_html(editor):
	"""
	Updates image size of &lt;img src=""&gt; tag
	@type editor: ZenEditor
	"""
	editor_file = editor.get_file_path()
	caret_pos = editor.get_caret_pos()
		
	if editor_file is None:
		raise zencoding.utils.ZenError("You should save your file before using this action")
		
	image = _find_image(editor)
	
	if image:
		# search for image path
		m = re.search(r'src=(["\'])(.+?)\1', image['tag'], re.IGNORECASE)
		if m:
			src = m.group(2)
			
		if src:
			size = get_image_size_for_source(editor, src)
			if size:
				new_tag = _replace_or_append(image['tag'], 'width', size['width'])
				new_tag = _replace_or_append(new_tag, 'height', size['height'])
				
				return {
					'data': new_tag,
					'start': image['start'],
					'end': image['end'],
					'caret': caret_pos
				}
	return False

def get_image_size_for_source(editor, src):
	"""
	Returns image dimentions for source
	@param {zen_editor} editor
	@param {String} src Image source (path or data:url)
	"""
	if src:
		# check if it is data:url
		if starts_with('data:', src):
			f_content = base64.b64decode( re.sub(r'^data\:.+?;.+?,', '', src) )
		else:
			editor_file = editor.get_file_path()
			
			if editor_file is None:
				raise zencoding.utils.ZenError("You should save your file before using this action")
			
			abs_src = zen_file.locate_file(editor_file, src)
			if not abs_src:
				raise zencoding.utils.ZenError("Can't locate '%s' file" % src)
			
			f_content = zen_file.read(abs_src)
		
		return zencoding.utils.get_image_size(f_content)

def _replace_or_append(img_tag, attr_name, attr_value):
	"""
	Replaces or adds attribute to the tag
	@type img_tag: str
	@type attr_name: str
	@type attr_value: str
	"""
	if attr_name in img_tag.lower():
		# attribute exists
		re_attr = re.compile(attr_name + r'=([\'"])(.*?)\1', re.I)
		return re.sub(re_attr, lambda m: '%s=%s%s%s' % (attr_name, m.group(1), attr_value, m.group(1)), img_tag)
	else:
		return re.sub(r'\s*(\/?>)$', ' %s="%s" \\1' % (attr_name, attr_value), img_tag)

def _find_image(editor):
	"""
	Find image tag under caret
 	@return Image tag and its indexes inside editor source
	"""
	_caret = editor.get_caret_pos()
	text = editor.get_content()
	start_ix = -1
	end_ix = -1
	
	# find the beginning of the tag
	caret_pos = _caret
	while caret_pos >= 0:
		if text[caret_pos] == '<':
			if text[caret_pos:caret_pos + 4].lower() == '<img':
				# found the beginning of the image tag
				start_ix = caret_pos
				break
			else:
				# found some other tag
				return None
		caret_pos -= 1
			
	# find the end of the tag 
	caret_pos = _caret
	ln = len(text)
	while caret_pos <= ln:
		if text[caret_pos] == '>':
			end_ix = caret_pos + 1
			break
		caret_pos += 1
	
	
	if start_ix != -1 and end_ix != -1:
		return {
			'start': start_ix,
			'end': end_ix,
			'tag': text[start_ix:end_ix]
		}
	
	return None

def find_css_insertion_point(tokens, start_ix):
	"""
	Search for insertion point for new CSS properties
	@param tokens: List of parsed CSS tokens
	@param start_ix: Token index where to start searching
	"""
	ins_point = None
	ins_ix = -1 
	need_col = False
	
	for i in range(start_ix, len(tokens)):
		t = tokens[i]
		if t['type'] == 'value':
			ins_point = t
			ins_ix = i
			
			# look ahead for rule termination
			if i + 1 < len(tokens) and tokens[i + 1]['type'] == ';':
				ins_point = tokens[i + 1]
				ins_ix += 1
			else:
				need_col = True
				
			break
	
	return {
		'token': ins_point,
		'ix': ins_ix,
		'need_col': need_col
	}

def update_image_size_css(editor):
	"""
	Updates image size of CSS rule
 	@type editor: ZenEditor
	"""
	caret_pos = editor.get_caret_pos()
	content = editor.get_content()
	rule = parser_utils.extract_css_rule(content, caret_pos, True)
	
	if rule:
		css = parser_utils.parse_css(content[rule[0]:rule[1]], rule[0])
		cur_token = find_token_from_position(css, caret_pos, 'identifier')
		value = find_value_token(css, cur_token + 1)
			
		if not value: return False
		
		# find insertion point
		ins_point = find_css_insertion_point(css, cur_token)
		
		m = re.match(r'url\((["\']?)(.+?)\1\)', value['content'], re.I)
		if m:
			size = get_image_size_for_source(editor, m.group(2))
			if size:
				wh = {'width': None, 'height': None}
				updates = []
				styler = learn_css_style(css, cur_token)
				
				for i, item in enumerate(css):
					if item['type'] == 'identifier' and item['content'] in wh:
						wh[item['content']] = i
					
				def update(name, val):
					v = None
					if wh[name] is not None:
						v = find_value_token(css, wh[name] + 1)
						
					if v:
						updates.append([v['start'], v['end'], '%spx' % val])
					else:
						updates.append([ins_point['token']['end'], ins_point['token']['end'], styler(name, '%spx' % val)])
						
				
				update('width', size['width'])
				update('height', size['height'])
				
				if updates:
					updates.sort(lambda a,b: a[0] - b[0])
#					updates = sorted(updates, key=lambda a: a[0]) 
					
					# some editors do not provide easy way to replace multiple code 
					# fragments so we have to squash all replace operations into one
					offset = updates[0][0]
					offset_end = updates[-1][1]
					data = content[offset:offset_end]
					
					updates.reverse()
					for u in updates:
						data = replace_substring(data, u[0] - offset, u[1] - offset, u[2])
							
						# also calculate new caret position
						if u[0] < caret_pos:
							caret_pos += len(u[2]) - u[1] + u[0]
						
					
					if ins_point['need_col']:
						data = replace_substring(data, ins_point['token']['end'] - offset, ins_point['token']['end'] - offset, ';')
					
					return {
						'data': data,
						'start': offset,
						'end': offset_end,
						'caret': caret_pos
					};
					
	return None

def learn_css_style(tokens, pos):
	"""
	Learns formatting style from parsed tokens
	@param tokens: List of tokens
	@param pos: Identifier token position, from which style should be learned
	@returns: Function with <code>(name, value)</code> arguments that will create
	CSS rule based on learned formatting
	"""
	prefix = ''
	glue = ''
	
	# use original tokens instead of optimized ones
	pos = tokens[pos]['ref_start_ix']
	tokens = tokens.original
	
	# learn prefix
	for i in xrange(pos - 1, -1, -1):
		if tokens[i]['type'] == 'white':
			prefix = tokens[i]['content'] + prefix
		elif tokens[i]['type'] == 'line':
			prefix = tokens[i]['content'] + prefix
			break
		else:  
			break
	
	# learn glue
	for t in tokens[pos+1:]:
		if t['type'] == 'white' or t['type'] == ':':
			glue += t['content']
		else:
			break
	
	if ':' not in glue:
		glue = ':'
		
	return lambda name, value: "%s%s%s%s;" % (prefix, name, glue, value)
	

def do_css_reflection(editor):
	content = editor.get_content()
	caret_pos = editor.get_caret_pos()
	css = parser_utils.extract_css_rule(content, caret_pos)
		
	if not css or caret_pos < css[0] or caret_pos > css[1]:
		# no matching CSS rule or caret outside rule bounds
		return False
		
	tokens = parser_utils.parse_css(content[css[0]:css[1]], css[0])
	token_ix = find_token_from_position(tokens, caret_pos, 'identifier')
	
	if token_ix != -1:
		cur_prop = tokens[token_ix]['content']
		value_token = find_value_token(tokens, token_ix + 1)
		base_name = get_base_css_name(cur_prop)
		re_name = re.compile('^(?:\\-\\w+\\-)?' + base_name + '$')
		re_name = get_reflected_css_name(base_name)
		values = []
			
		if not value_token:
			return False
			
		# search for all vendor-prefixed properties
		for i, token in enumerate(tokens):
			if token['type'] == 'identifier' and re.search(re_name, token['content']) and token['content'] != cur_prop:
				v = find_value_token(tokens, i + 1)
				if v:
					values.append({'name': token, 'value': v})
		
		# some editors do not provide easy way to replace multiple code 
		# fragments so we have to squash all replace operations into one
		if values:
			data = content[values[0]['value']['start']:values[-1]['value']['end']]
			offset = values[0]['value']['start']
			value = value_token['content']
				
			values.reverse()
			for v in values:
				rv = get_reflected_value(cur_prop, value, v['name']['content'], v['value']['content'])
				data = replace_substring(data, v['value']['start'] - offset, v['value']['end'] - offset, rv)
					
				# also calculate new caret position
				if v['value']['start'] < caret_pos:
					caret_pos += len(rv) - len(v['value']['content'])
				
			return {
				'data': data,
				'start': offset,
				'end': values[0]['value']['end'],
				'caret': caret_pos
			}
	
	return None

def get_base_css_name(name):
	"""
    Removes vendor prefix from CSS property
    @param name: CSS property
    @type name: str
    @return: str
	"""
	return re.sub(r'^\s*\-\w+\-', '', name)

def get_reflected_css_name(name):
	"""
    Returns regexp that should match reflected CSS property names
    @param name: Current CSS property name
    @type name: str
    @return: RegExp
	"""
	name = get_base_css_name(name)
	vendor_prefix = '^(?:\\-\\w+\\-)?'
	
	if name == 'opacity' or name == 'filter':
		return re.compile(vendor_prefix + '(?:opacity|filter)$')
	
	m = re.match(r'^border-radius-(top|bottom)(left|right)', name)
	if m:
		# Mozilla-style border radius
		return re.compile(vendor_prefix + '(?:%s|border-%s-%s-radius)$' % (name, m.group(1), m.group(2)) )
	
	m = re.match(r'^border-(top|bottom)-(left|right)-radius', name)
	if m: 
		return re.compile(vendor_prefix + '(?:%s|border-radius-%s%s)$'  % (name, m.group(1), m.group(2)) );
	
	return re.compile(vendor_prefix + name + '$')

def get_reflected_value(cur_name, cur_value, ref_name, ref_value):
	"""
    Returns value that should be reflected for <code>ref_name</code> CSS property
    from <code>cur_name</code> property. This function is used for special cases,
    when the same result must be achieved with different properties for different
    browsers. For example: opаcity:0.5; -> filter:alpha(opacity=50);<br><br>
     
    This function does value conversion between different CSS properties
     
    @param cur_name: Current CSS property name
    @type cur_name: str
    @param cur_value: Current CSS property value
    @type cur_value: str
    @param ref_name: Receiver CSS property's name 
    @type ref_name: str 
    @param ref_value: Receiver CSS property's value
    @type ref_value: str
    @return: New value for receiver property
	"""
	cur_name = get_base_css_name(cur_name)
	ref_name = get_base_css_name(ref_name)
	
	if cur_name == 'opacity' and ref_name == 'filter':
		return re.sub(re.compile(r'opacity=[^\)]*', re.IGNORECASE), 'opacity=' + math.floor(float(cur_value) * 100), ref_value)
	if cur_name == 'filter' and ref_name == 'opacity':
		m = re.search(r'opacity=([^\)]*)', cur_value, re.IGNORECASE)
		return prettify_number(int(m.group(1)) / 100) if m else ref_value
	
	
	return cur_value

def find_value_token(tokens, pos):
	"""
    Find value token, staring at <code>pos</code> index and moving right
    @type tokens: list
    @type pos: int
    @return: token
	"""
	for t in tokens[pos:]:
		if t['type'] == 'value':
			return t
		elif t['type'] == 'identifier' or t['type'] == ';':
			break
		
	return None

def replace_substring(text, start, end, new_value):
	"""
    Replace substring of <code>text</code>, defined by <code>start</code> and 
    <code>end</code> indexes with <code>new_value</code>
    @type text: str
    @type start: int
    @type end: int
    @type new_value: str
    @return: str
	"""
	return text[0:start] + new_value + text[end:]

def find_token_from_position(tokens, pos, type):
	"""
    Search for token with specified type left to the specified position
    @param tokens: List of parsed tokens
    @type tokens: list
    @param pos: Position where to start searching
    @type pos: int
    @param type: Token type
    @type type: str
    @return: Token index
	"""
	# find token under caret
	token_ix = -1;
	for i, token in enumerate(tokens):
		if token['start'] <= pos and token['end'] >= pos:
			token_ix = i
			break
		
	if token_ix != -1:
		# token found, search left until we find token with specified type
		while token_ix >= 0:
			if tokens[token_ix]['type'] == type:
				return token_ix
			token_ix -= 1
	
	return -1



'''
A Python port of Stoyan Stefanov's CSSEX CSS parser

How to use:
call <code>lex(source)</code> to parse CSS source into tokens
call <code>to_source(tokens)</code> to transform parsed tokens back to CSS source

@link https://github.com/stoyan/etc/tree/master/cssex
@author: Sergey Chikuyonok

'''

class Walker(object):
	def __init__(self, source=None):
		self.lines = None
		self.total_lines = 0
		self.line = ''
		self.ch =  ''
		self.linenum = -1
		self.chnum = -1

		if source is not None:
			self.init(source)

	def init(self, source):
		# source, yumm
		self.lines = source.splitlines()
		self.total_lines = len(self.lines)

		# reset
		self.chnum = -1
		self.linenum = -1
		self.ch = ''
		self.line = ''

		# advance
		self.next_line()
		self.next_char()


	def next_line(self):
		self.linenum += 1
		if self.total_lines <= self.linenum:
			self.line = False
		else:
			self.line = self.lines[self.linenum]

		if self.chnum != -1:
			self.chnum = 0

		return self.line

	def next_char(self):
		self.chnum += 1
		while get_char(self.line, self.chnum) is None:
			if self.next_line() is False:
				self.ch = False
				return False # end of source

			self.chnum = -1
			self.ch = '\n'
			return '\n'

		self.ch = self.line[self.chnum]
		return self.ch

_walker = Walker()
__tokens = []

# utility helpers
def get_char(text, pos):
	if pos >= 0 and pos < len(text):
		return text[pos]
	else:
		return None

def is_name_char(c):
	return c == '_' or c == '-' or c.isalpha()

def is_op(ch, matchattr=None):
	if matchattr:
		return ch in '*^|$~'

	return ch in "{}[]()+*=.,;:>~|\\%$#@^!"

def get_conf():
	return {
		'char': _walker.chnum,
		'line': _walker.linenum
	}

def tokener(value, token_type=None, c={}):
	"creates token objects and pushes them to a list"
	w = _walker
	__tokens.append({
		'charstart': c.get('char', w.chnum),
		'charend':   c.get('charend', w.chnum),
		'linestart': c.get('line', w.linenum),
		'lineend':   c.get('lineend', w.linenum),
		'value':     value,
		'type':      token_type or value
	})


# oops
class CSSEXError(Exception):
	def __init__(self, value, conf=None):
		self.value = value
		self.conf = conf or {}
		self.w = _walker

	def __str__(self):
		c = 'char' in self.conf and self.conf['char'] or self.w.chnum
		l = 'line' in self.conf and self.conf['line'] or self.w.linenum
		return "%s at line %d char %d" % (self.value, l + 1, c + 1)


# token handlers follow for:
# white space, comment, string, identifier, number, operator
def white():
	c = _walker.ch
	token = ''
	conf = get_conf()

	while c == " " or c == "\t":
		token += c
		c = _walker.next_char()


	tokener(token, 'white', conf)

def comment():
	w = _walker
	c = w.ch
	token = c
	conf = get_conf()

	cnext = w.next_char()

	if cnext != '*':
		# oops, not a comment, just a /
		conf['charend'] = conf['char']
		conf['lineend'] = conf['line']
		return tokener(token, token, conf)

	while not (c == "*" and cnext == "/"):
		token += cnext
		c = cnext
		cnext = w.next_char()

	token += cnext
	w.next_char()
	tokener(token, 'comment', conf)

def str():
	w = _walker
	c = w.ch
	q = c
	token = c
	conf = get_conf()

	c = w.next_char()

	while c != q:

		if c == '\n':
			cnext = w.next_char()
			if cnext == "\\":
				token += c + cnext
			else:
				# end of line with no \ escape = bad
				raise CSSEXError("Unterminated string", conf)

		else:
			if c == "\\":
				token += c + w.next_char()
			else:
				token += c

		c = w.next_char()

	token += c
	w.next_char()
	tokener(token, 'string', conf)

def brace():
	w = _walker
	c = w.ch
	depth = 0
	token = c
	conf = get_conf()

	c = w.next_char()

	while c != ')' and not depth:
		if c == '(':
			depth += 1
		elif c == ')':
			depth -= 1
		elif c is False:
			raise CSSEXError("Unterminated brace", conf)

		token += c
		c = w.next_char()

	token += c
	w.next_char()
	tokener(token, 'brace', conf)

def identifier(pre=None):
	w = _walker
	c = w.ch
	conf = get_conf()
	token = pre and pre + c or c

	c = w.next_char()

	if pre: # adjust token position
		conf['char'] -= len(pre)

	while is_name_char(c) or c.isdigit():
		token += c
		c = w.next_char()

	tokener(token, 'identifier', conf)

def num():
	w = _walker
	c = w.ch
	conf = get_conf()
	token = c
	point = token == '.'

	c = w.next_char()
	nondigit = not c.isdigit()

	# .2px or .classname?
	if point and nondigit:
		# meh, NaN, could be a class name, so it's an operator for now
		conf['charend'] = conf['char']
		conf['lineend'] = conf['line']
		return tokener(token, '.', conf)

	# -2px or -moz-something
	if token == '-' and nondigit:
		return identifier('-')

	while c is not False and (c.isdigit() or (not point and c == '.')):  # not end of source && digit or first instance of .
		if c == '.':
			point = True

		token += c
		c = w.next_char()

	tokener(token, 'number', conf)

def op():
	w = _walker
	c = w.ch
	conf = get_conf()
	token = c
	next = w.next_char()

	if next == "=" and is_op(token, True):
		token += next
		tokener(token, 'match', conf)
		w.next_char()
		return

	conf['charend'] = conf['char'] + 1
	conf['lineend'] = conf['line']
	tokener(token, token, conf)

# call the appropriate handler based on the first character in a token suspect
def tokenize():
	ch = _walker.ch

	if ch == " " or ch == "\t":
		return white()

	if ch == '/':
		return comment()

	if ch == '"' or ch == "'":
		return str()
	
	if ch == '(':
		return brace()

	if ch == '-' or ch == '.' or ch.isdigit(): # tricky - char: minus (-1px) or dash (-moz-stuff)
		return num()

	if is_name_char(ch):
		return identifier()

	if is_op(ch):
		return op()

	if ch == "\n":
		tokener("line")
		_walker.next_char()
		return

	raise CSSEXError("Unrecognized character")

def parse(source):
	"""
	Parse CSS source
	@type source: str
	"""
	_walker.init(source)
	globals()['__tokens'] = []
	while _walker.ch is not False:
		tokenize()

	return __tokens


def to_source(tokens):
	"""
	Transform parsed tokens to CSS source
	@type tokens: list
	"""
	src = ''
	for t in tokens:
		if t['type'] == 'line':
			src += '\n'
		else:
			src += t['value']

	return src




'''
This file defines an XML parser, with a few kludges to make it
usable for HTML. autoSelfClosers defines a set of tag names that
are expected to not have a closing tag, and doNotIndent specifies
the tags inside of which no indentation should happen (see Config
object). These can be disabled by passing the editor an object like
{useHTMLKludges: false} as parserConfig option.

Original code by Marijn Haverbeke
from CodeMirror project: http://codemirror.net/

=====
Run <code>parse(text)</code> method to parse HTML string 
=====
'''
import re

class StopIteration(Exception):
	def __str__(self):
		return 'StopIteration'

def is_white_space(ch):
	return ch != "\n" and ch.isspace()

class Tokenizer(object):
	def __init__(self, source, state):
		self.state = state
		self.source = source

	def take(self, type):
		if isinstance(type, basestring):
			type = {
				'style' : type,
				'type' : type
			}

		if 'content' not in type:
			type['content'] = ''

		type['content'] += self.source.get()
		if not re.search(r'\n$', type['content']):
			self.source.next_while(is_white_space)

		type['value'] = type['content'] + self.source.get()
		return type

	def next(self):
		if not self.source.more():
			raise StopIteration

		if self.source.equals("\n"):
			self.source.next()
			return self.take("whitespace")

		type = None
		if self.source.applies(is_white_space):
			type = "whitespace"
		else:
			def save(s):
				self.state = s

			while not type:
				type = self.state(self.source, save)

		return self.take(type)

class TraverseDOM(object):
	def __init__(self, source):
		self.source = source
		self.__fed = False

	def next(self):
		if not self.__fed:
			self.__fed = True
			return self.source
		else:
			raise StopIteration

class StringStream(object):
	"""
	String streams are the things fed to parsers (which can feed them to a
	tokenizer if they want). They provide peek and next methods for looking at
	the current character (next 'consumes' this character, peek does not), and a
	get method for retrieving all the text that was consumed since the last time
	get was called.

	An easy mistake to make is to let a StopIteration exception finish the token
	stream while there are still characters pending in the string stream (hitting
	the end of the buffer while parsing a token). To make it easier to detect
	such errors, the stringstreams throw an exception when this happens.
	"""

	def __init__(self, source):
		"""
		Make a stringstream stream out of an iterator that returns strings.
		This is applied to the result of traverseDOM (see codemirror.js),
		and the resulting stream is fed to the parser.
		"""
		self.source = source
		self.current = ''
		"String that's currently being iterated over."

		self.pos = 0
		"Position in that string."

		self.accum = ""
		"Accumulator for strings that have been iterated over but not get()-ed yet."

		# ZC fix: if we've passed a string, wrap it with traverseDOM-like interface
		if isinstance(source, basestring):
			self.source = TraverseDOM(source)

	def ensure_chars(self):
		"Make sure there are more characters ready, or throw StopIteration."
		while self.pos == len(self.current):
			self.accum += self.current
			self.current = "" # In case source.next() throws
			self.pos = 0
			try:
				self.current = self.source.next()
			except StopIteration:
				return False

		return True

	def peek(self):
		"Return the next character in the stream."
		if not self.ensure_chars():
			return None

		return self.current[self.pos]

	def next(self):
		"""
		Get the next character, throw StopIteration if at end, check
		for unused content.
		"""
		if not self.ensure_chars():
			if len(self.accum) > 0:
				raise "End of stringstream reached without emptying buffer ('" + self.accum + "')."
			else:
				raise StopIteration

		result = self.pos < len(self.current) and self.current[self.pos] or None
		self.pos += 1
		return result

	def get(self):
		"Return the characters iterated over since the last call to .get()."
		temp = self.accum
		self.accum = ""
		if self.pos > 0:
			temp += self.current[0:self.pos]
			self.current = self.current[self.pos:]
			self.pos = 0

		return temp

	def push(self, str):
		"Push a string back into the stream."
		self.current = self.current[0:self.pos] + str + self.current[self.pos:]

	def look_ahead(self, str, consume, skip_spaces, case_insensitive):
		def cased(str):
			return case_insensitive and str.lower() or str

		str = cased(str)
		found = False

		_accum = self.accum
		_pos = self.pos
		if skip_spaces:
			self.next_while_matches(r'\s')

		while True:
			end = self.pos + len(str)
			left = len(self.current) - self.pos

			if end <= len(self.current):
				found = str == cased(self.current[self.pos:end])
				self.pos = end
				break
			elif str[0:left] == cased(self.current[self.pos:]):
				self.accum += self.current
				self.current = ""
				try:
					self.current = self.source.next()
				except StopIteration:
					break

				self.pos = 0
				str = str[left:]
			else:
				break

		if not (found and consume):
			self.current = self.accum[len(_accum):] + self.current
			self.pos = _pos
			self.accum = _accum


		return found

	def look_ahead_regex(self, regex, consume):
		"Wont't match past end of line."
		if regex[0] != "^":
			raise Exception("Regexps passed to lookAheadRegex must start with ^")

		# Fetch the rest of the line
		while self.current.find("\n", self.pos) == -1:
			try:
				self.current += self.source.next()
			except StopIteration:
				break

		matched = re.match(regex, self.current[self.pos:])
		if matched and consume:
			self.pos += len(matched.group(0))

		return matched

	def more(self):
		"Produce true if the stream isn't empty."
		return self.peek() is not None

	def applies(self, test):
		next = self.peek()
		return next is not None and test(next)

	def next_while(self, test):
		next = self.peek()
		while next is not None and test(next):
			self.next()
			next = self.peek()

	def matches(self, regexp):
		next = self.peek()
		return next is not None and re.search(regexp, next)

	def next_while_matches(self, regexp):
		next = self.peek()
		while next is not None and re.search(regexp, next):
			self.next()
			next = self.peek()

	def equals(self, ch):
		return ch == self.peek()

	def end_of_line(self):
		next = self.peek()
		return next is None or next == "\n"

Kludges = {
'autoSelfClosers': {"br": True, "img": True, "hr": True, "link": True, "input": True,
					"meta": True, "col": True, "frame": True, "base": True, "area": True},
'doNotIndent': {"pre": True, "!cdata": True}
}
NoKludges = {'autoSelfClosers': {}, 'doNotIndent': {"!cdata": True}}
UseKludges = Kludges
alignCDATA = False

def tokenize_xml(source, start_state=None):
	"""
	Simple stateful tokenizer for XML documents. Returns a
	MochiKit-style iterator, with a state property that contains a
	function encapsulating the current state. See tokenize.js.
	"""

	def in_text(source, set_state):
		ch = source.next()
		if ch == "<":
			if source.equals("!"):
				source.next();
				if source.equals("["):
					if source.look_ahead("[CDATA[", True):
						set_state(in_block("xml-cdata", "]]>"))
						return None
					else:
						return "xml-text"
				elif source.look_ahead("--", True):
					set_state(in_block("xml-comment", "-->"))
					return None
				elif source.look_ahead("DOCTYPE", True):
					source.next_while_matches(r'[\w\._\-]')
					set_state(in_block("xml-doctype", ">"))
					return "xml-doctype"
				else:
					return "xml-text"
			elif source.equals("?"):
				source.next()
				source.next_while_matches(r'[\w\._\-]')
				set_state(in_block("xml-processing", "?>"))
				return "xml-processing"
			else:
				if source.equals("/"):
					source.next()
				set_state(in_tag)
				return "xml-punctuation"
		elif ch == "&":
			while not source.end_of_line():
				if source.next() == ";":
					break;
			return "xml-entity";
		else:
			source.next_while_matches(r'[^&<\n]')
			return "xml-text";

	def in_tag(source, set_state):
		ch = source.next()
		if ch == ">":
			set_state(in_text)
			return "xml-punctuation";
		elif re.match(r'[?\/]', ch) and source.equals(">"):
			source.next();
			set_state(in_text)
			return "xml-punctuation"
		elif ch == "=":
			return "xml-punctuation"
		elif re.match(r'[\'"]', ch):
			set_state(in_attribute(ch))
			return None
		else:
			source.next_while_matches(r'[^\s=<>\"\'\/?]')
			return "xml-name"

	def in_attribute(quote):
		def fn(source, set_state):
			while not source.end_of_line():
				if source.next() == quote:
					set_state(in_tag)
					break
			return "xml-attribute";

		return fn

	def in_block(style, terminator):
		def fn(source, set_state):
			while not source.end_of_line():
				if source.look_ahead(terminator, True):
					set_state(in_text)
					break
				source.next()

			return style

		return fn

	return Tokenizer(source, start_state or in_text)

def parse(source):
	"Parse HTML source"
	if isinstance(source, basestring):
		source = StringStream(source)
	
	tokens = tokenize_xml(source)
	token = [None]
	current_tag = None
	context = [None]
	consume = [False]

	def push(fs):
		i = len(fs) - 1
		while i >= 0:
			cc.append(fs[i])
			i -= 1

	def cont(*args):
		push(args);
		consume[0] = True

	def _pass(*args):
		push(args)
		consume[0] = False

	def mark_err():
		token[0]['style'] += " xml-error"

	def expect(text):
		def fn(style, content):
			if content == text: cont()
			else: mark_err()

		return fn

	def push_context(tagname):
		context[0] = {
			'prev': context[0],
			'name': tagname
		}

	def pop_context():
		context[0] = context[0]['prev']
		
	harmless_tokens = {
		"xml-text": True,
		"xml-entity": True,
		"xml-comment": True,
		"xml-processing": True,
		"xml-doctype": True
	}

	def base(*args):
		return _pass(element, base)
	
	def element(style, content):
		if content == "<":
			cont(tagname, attributes, endtag())
		elif content == "</":
			cont(closetagname, expect(">"))
		elif style == "xml-cdata":
			if not context[0] or context[0]['name'] != "!cdata":
				push_context("!cdata")
			if re.search(r'\]\]>$', content):
				pop_context()
			cont()
		elif style in harmless_tokens:
			cont()
		else:
			mark_err()
			cont()

	def tagname(style, content):
		if style == "xml-name":
			current_tag = content.lower()
			token[0]['style'] = "xml-tagname"
			cont()
		else:
			current_tag = None
			_pass()

	def closetagname(style, content):
		if style == "xml-name":
			token[0]['style'] = "xml-tagname"
			if context[0] and content.lower() == context[0]['name']:
				pop_context()
			else:
				mark_err()
		cont()

	def endtag(*args):
		def fn(style, content):
			if content == "/>" or (content == ">" and current_tag in UseKludges['autoSelfClosers']):
				cont()
			elif content == ">":
				push_context(current_tag)
				cont()
			else: 
				mark_err()
				cont()
				
		return fn
	
	def attributes(style, content):
		if style == "xml-name":
			token[0]['style'] = "xml-attname"
			cont(attribute, attributes)
		else:
			_pass()
	
	def attribute(style, content):
		if content == "=":
			cont(value)
		elif content == ">" or content == "/>":
			_pass(endtag)
		else:
			_pass()
	
	def value(style, content):
		if style == "xml-attribute":
			cont(value)
		else: 
			_pass()
	
	def next():
		token[0] = tokens.next()
		if token[0]['style'] == "whitespace" or token[0]['type'] == "xml-comment":
			return token[0]

		while True:
			consume[0] = False
			cc.pop()(token[0]['style'], token[0]['content'])
			if consume[0]: 
				return token[0]
			
	cc = [base]
	return {'next': next}


	



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Process CSS properties: replaces snippets, augumented with ! char, with 
<em>!important</em> suffix
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re
import zencoding

re_important = re.compile(r'(.+)\!$')

@zencoding.filter('css')
def process(tree, profile):
	for item in tree.children:
		# CSS properties are always snippets
		if item.type == 'snippet' and re_important.search(item.real_name):
			item.start = re.sub(r'(;?)$', ' !important\\1', item.start)
		
		process(item, profile)
	
	return tree


'''
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import os.path

def read(path):
	"""
	Read file content and return it
	@param path: File's relative or absolute path
	@type path: str
	@return: str
	"""
	content = None
	try:
		fp = open(path, 'rb')
		content = fp.read()
		fp.close()
	except:
		pass
	
	return content

def locate_file(editor_file, file_name):
	"""
	Locate <code>file_name</code> file that relates to <code>editor_file</code>.
	File name may be absolute or relative path
	
	@type editor_file: str
	@type file_name: str
	@return String or None if <code>file_name</code> cannot be located
	"""
	result = None
	
	previous_parent = ''
	parent = os.path.dirname(editor_file)
	while parent and os.path.exists(parent) and parent != previous_parent:
		tmp = create_path(parent, file_name)
		if os.path.exists(tmp):
			result = tmp
			break
		
		previous_parent = parent
		parent = os.path.dirname(parent)
	
	return result

def create_path(parent, file_name):
	"""
	Creates absolute path by concatenating <code>parent</code> and <code>file_name</code>.
	If <code>parent</code> points to file, its parent directory is used
	
	@type parent: str
	@type file_name: str
	@return: str
	"""
	result = ''
	file_name = file_name.lstrip('/')
	
	if os.path.exists(parent):
		if os.path.isfile(parent):
			parent = os.path.dirname(parent)
			
		result = os.path.normpath(os.path.join(parent, file_name))
	
	return result

def save(file, content):
	"""
	Saves <code>content</code> as <code>file</code>
	
	@param file: File's asolute path
	@type file: str
	@param content: File content
	@type content: str
	"""
	try:
		fp = open(file, 'wb')
	except:
		fdirs, fname = os.path.split(file)
		if fdirs:
			os.makedirs(fdirs)
		fp = open(file, 'wb')
		
	fp.write(content)
	fp.close()

def get_ext(file):
	"""
	Returns file extention in lower case
	@type file: str
	@return: str
	"""
	ext = os.path.splitext(file)[1]
	if ext:
		ext = ext[1:]
	
	return ext.lower()



import zencoding.utils as utils
import re
import os

__actions = {}
__filters = {}
__imported = []

def action(name=None, action_func=None):
	"Decorator for Zen Coding actions"
	if name is None and action_func is None:
		# @zencoding.action()
		return action_function
	elif action_func is None:
		if(callable(name)):
			# @zencoding.action
			return action_function(name)
		else:
			# @zencoding.action('somename') or @zencoding.action(name='somename')
			def dec(func):
				return action(name, func)
			return dec
	elif name is not None and action_func is not None:
		# zencoding.action('somename', somefunc)
		__actions[name] = action_func
		return action_func
	else:
		raise ValueError("Unsupported arguments to Zen Action: (%r, %r)", (name, action_func))

def action_function(func):
	__actions[getattr(func, "_decorated_function", func).__name__] = func
	return func

def filter(name=None, filter_func=None):
	"Decorator for Zen Coding filters"
	if name == None and filter_func == None:
		# @zencoding.filter()
		return filter_function
	elif filter_func == None:
		if(callable(name)):
			# @zencoding.filter
			return filter_function(name)
		else:
			# @zencoding.filter('somename') or @zencoding.filter(name='somename')
			def dec(func):
				return filter(name, func)
			return dec
	elif name != None and filter_func != None:
		# zencoding.filter('somename', somefunc)
		__filters[name] = filter_func
		return filter_func
	else:
		raise ValueError("Unsupported arguments to Zen Filter: (%r, %r)", (name, filter_func))

def filter_function(func):
	__filters[getattr(func, "_decorated_function", func).__name__] = func
	return func

def run_action(name, *args, **kwargs):
	"""
	 Runs Zen Coding action. For list of available actions and their
	 arguments see zen_actions.py file.
	 @param name: Action name 
	 @type name: str 
	 @param args: Additional arguments. It may be array of arguments
	 or inline arguments. The first argument should be <code>zen_editor</code> instance
	 @type args: list
	 @example
	 zencoding.run_actions('expand_abbreviation', zen_editor)
	 zencoding.run_actions('wrap_with_abbreviation', zen_editor, 'div')  
	"""
	import zencoding.actions
	if name in __actions:
		return __actions[name](*args, **kwargs)
	
	return False
		
def run_filters(tree, profile, filter_list):
	"""
	Runs filters on tree
	@type tree: ZenNode
	@param profile: str, object
	@param filter_list: str, list
	@return: ZenNode
	"""
	import zencoding.filters
	
	profile = utils.process_profile(profile)
		
	if isinstance(filter_list, str):
		filter_list = re.split(r'[\|,]', filter_list)
		
	for name in filter_list:
		name = name.strip()
		if name and name in __filters:
			tree = __filters[name](tree, profile)
			
	return tree

def expand_abbreviation(abbr, syntax='html', profile_name='plain'):
	"""
	Expands abbreviation into a XHTML tag string
	@type abbr: str
	@return: str
	"""
	tree_root = utils.parse_into_tree(abbr, syntax)
	if tree_root:
		tree = utils.rollout_tree(tree_root)
		utils.apply_filters(tree, syntax, profile_name, tree_root.filters)
		return utils.replace_variables(tree.to_string())
	
	return ''

def wrap_with_abbreviation(abbr, text, syntax='html', profile='plain'):
	"""
	Wraps passed text with abbreviation. Text will be placed inside last
	expanded element
	@param abbr: Abbreviation
	@type abbr: str
	
	@param text: Text to wrap
	@type text: str
	
	@param syntax: Document type (html, xml, etc.)
	@type syntax: str
	
	@param profile: Output profile's name.
	@type profile: str
	@return {String}
	"""
	tree_root = utils.parse_into_tree(abbr, syntax)
	pasted = False
	
	if tree_root:
		if tree_root.multiply_elem:
			# we have a repeating element, put content in
			tree_root.multiply_elem.set_paste_content(text)
			tree_root.multiply_elem.repeat_by_lines = pasted = True
			
		
		tree = utils.rollout_tree(tree_root)
		
		if not pasted:
			tree.paste_content(text)
		
		utils.apply_filters(tree, syntax, profile, tree_root.filters)
		return utils.replace_variables(tree.to_string())
	
	return None


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""`cssmin` - A Python port of the YUI CSS compressor."""

try:
    from StringIO import StringIO # The pure-Python StringIO supports unicode.
except ImportError:
    from io import StringIO
import re


__version__ = '0.2.0'


def remove_comments(css):
    """Remove all CSS comment blocks."""

    iemac = False
    preserve = False
    comment_start = css.find("/*")
    while comment_start >= 0:
        # Preserve comments that look like `/*!...*/`.
        # Slicing is used to make sure we don"t get an IndexError.
        preserve = css[comment_start + 2:comment_start + 3] == "!"

        comment_end = css.find("*/", comment_start + 2)
        if comment_end < 0:
            if not preserve:
                css = css[:comment_start]
                break
        elif comment_end >= (comment_start + 2):
            if css[comment_end - 1] == "\\":
                # This is an IE Mac-specific comment; leave this one and the
                # following one alone.
                comment_start = comment_end + 2
                iemac = True
            elif iemac:
                comment_start = comment_end + 2
                iemac = False
            elif not preserve:
                css = css[:comment_start] + css[comment_end + 2:]
            else:
                comment_start = comment_end + 2
        comment_start = css.find("/*", comment_start)

    return css


def remove_unnecessary_whitespace(css):
    """Remove unnecessary whitespace characters."""

    def pseudoclasscolon(css):

        """
        Prevents 'p :link' from becoming 'p:link'.

        Translates 'p :link' into 'p ___PSEUDOCLASSCOLON___link'; this is
        translated back again later.
        """

        regex = re.compile(r"(^|\})(([^\{\:])+\:)+([^\{]*\{)")
        match = regex.search(css)
        while match:
            css = ''.join([
                css[:match.start()],
                match.group().replace(":", "___PSEUDOCLASSCOLON___"),
                css[match.end():]])
            match = regex.search(css)
        return css

    css = pseudoclasscolon(css)
    # Remove spaces from before things.
    css = re.sub(r"\s+([!{};:>+\(\)\],])", r"\1", css)

    # If there is a `@charset`, then only allow one, and move to the beginning.
    css = re.sub(r"^(.*)(@charset \"[^\"]*\";)", r"\2\1", css)
    css = re.sub(r"^(\s*@charset [^;]+;\s*)+", r"\1", css)

    # Put the space back in for a few cases, such as `@media screen` and
    # `(-webkit-min-device-pixel-ratio:0)`.
    css = re.sub(r"\band\(", "and (", css)

    # Put the colons back.
    css = css.replace('___PSEUDOCLASSCOLON___', ':')

    # Remove spaces from after things.
    css = re.sub(r"([!{}:;>+\(\[,])\s+", r"\1", css)

    return css


def remove_unnecessary_semicolons(css):
    """Remove unnecessary semicolons."""

    return re.sub(r";+\}", "}", css)


def remove_empty_rules(css):
    """Remove empty rules."""

    return re.sub(r"[^\}\{]+\{\}", "", css)


def normalize_rgb_colors_to_hex(css):
    """Convert `rgb(51,102,153)` to `#336699`."""

    regex = re.compile(r"rgb\s*\(\s*([0-9,\s]+)\s*\)")
    match = regex.search(css)
    while match:
        colors = map(lambda s: s.strip(), match.group(1).split(","))
        hexcolor = '#%.2x%.2x%.2x' % tuple(map(int, colors))
        css = css.replace(match.group(), hexcolor)
        match = regex.search(css)
    return css


def condense_zero_units(css):
    """Replace `0(px, em, %, etc)` with `0`."""

    return re.sub(r"([\s:])(0)(px|em|%|in|cm|mm|pc|pt|ex)", r"\1\2", css)


def condense_multidimensional_zeros(css):
    """Replace `:0 0 0 0;`, `:0 0 0;` etc. with `:0;`."""

    css = css.replace(":0 0 0 0;", ":0;")
    css = css.replace(":0 0 0;", ":0;")
    css = css.replace(":0 0;", ":0;")

    # Revert `background-position:0;` to the valid `background-position:0 0;`.
    css = css.replace("background-position:0;", "background-position:0 0;")

    return css


def condense_floating_points(css):
    """Replace `0.6` with `.6` where possible."""

    return re.sub(r"(:|\s)0+\.(\d+)", r"\1.\2", css)


def condense_hex_colors(css):
    """Shorten colors from #AABBCC to #ABC where possible."""

    regex = re.compile(r"([^\"'=\s])(\s*)#([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])")
    match = regex.search(css)
    while match:
        first = match.group(3) + match.group(5) + match.group(7)
        second = match.group(4) + match.group(6) + match.group(8)
        if first.lower() == second.lower():
            css = css.replace(match.group(), match.group(1) + match.group(2) + '#' + first)
            match = regex.search(css, match.end() - 3)
        else:
            match = regex.search(css, match.end())
    return css


def condense_whitespace(css):
    """Condense multiple adjacent whitespace characters into one."""

    return re.sub(r"\s+", " ", css)


def condense_semicolons(css):
    """Condense multiple adjacent semicolon characters into one."""

    return re.sub(r";;+", ";", css)


def wrap_css_lines(css, line_length):
    """Wrap the lines of the given CSS to an approximate length."""

    lines = []
    line_start = 0
    for i, char in enumerate(css):
        # It's safe to break after `}` characters.
        if char == '}' and (i - line_start >= line_length):
            lines.append(css[line_start:i + 1])
            line_start = i + 1

    if line_start < len(css):
        lines.append(css[line_start:])
    return '\n'.join(lines)


def cssmin(css, wrap=None):
    css = remove_comments(css)
    css = condense_whitespace(css)
    # A pseudo class for the Box Model Hack
    # (see http://tantek.com/CSS/Examples/boxmodelhack.html)
    css = css.replace('"\\"}\\""', "___PSEUDOCLASSBMH___")
    css = remove_unnecessary_whitespace(css)
    css = remove_unnecessary_semicolons(css)
    css = condense_zero_units(css)
    css = condense_multidimensional_zeros(css)
    css = condense_floating_points(css)
    css = normalize_rgb_colors_to_hex(css)
    css = condense_hex_colors(css)
    if wrap is not None:
        css = wrap_css_lines(css, wrap)
    css = css.replace("___PSEUDOCLASSBMH___", '"\\"}\\""')
    css = condense_semicolons(css)
    return css.strip()


def main():
    import optparse
    import sys

    p = optparse.OptionParser(
        prog="cssmin", version=__version__,
        usage="%prog [--wrap N]",
        description="""Reads raw CSS from stdin, and writes compressed CSS to stdout.""")

    p.add_option(
        '-w', '--wrap', type='int', default=None, metavar='N',
        help="Wrap output to approximately N chars per line.")

    options, args = p.parse_args()
    sys.stdout.write(cssmin(sys.stdin.read(), wrap=options.wrap))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Comment important tags (with 'id' and 'class' attributes)
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import zencoding
import zencoding.utils as utils

def add_comments(node, i):
	
	"""
	Add comments to tag
	@type node: ZenNode
	@type i: int
	"""
	id_attr = node.get_attribute('id')
	class_attr = node.get_attribute('class')
	nl = utils.get_newline()
		
	if id_attr or class_attr:
		comment_str = ''
		padding = node.parent and node.parent.padding or ''
		if id_attr: comment_str += '#' + id_attr
		if class_attr: comment_str += '.' + class_attr
		
		node.start = node.start.replace('<', '<!-- ' + comment_str + ' -->' + nl + padding + '<', 1)
		node.end = node.end.replace('>', '>' + nl + padding + '<!-- /' + comment_str + ' -->', 1)
		
		# replace counters
		counter = zencoding.utils.get_counter_for_node(node)
		node.start = utils.replace_counter(node.start, counter)
		node.end = utils.replace_counter(node.end, counter)

@zencoding.filter('c')
def process(tree, profile):
	if profile['tag_nl'] is False:
		return tree
		
	for i, item in enumerate(tree.children):
		if item.is_block():
			add_comments(item, i)
		process(item, profile)
	
	return tree


'''
High-level editor interface that communicates with underlying editor (like
Espresso, Coda, etc.) or browser.
Basically, you should call <code>set_context(obj)</code> method to
set up undelying editor context before using any other method.

This interface is used by <i>zen_actions.py</i> for performing different
actions like <b>Expand abbreviation</b>

@example
from zencoding.interface.editor import ZenEditor
editor = ZenEditor(context)
//now you are ready to use editor object
editor.get_selection_range();

@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
class ZenEditor():
	def __init__(self):
		pass

	def set_context(self, context):
		"""
		Setup underlying editor context. You should call this method
		<code>before</code> using any Zen Coding action.
		@param context: context object
		"""
		pass

	def get_selection_range(self):
		"""
		Returns character indexes of selected text
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_selection_range();
		print('%s, %s' % (start, end))
		"""
		return 0, 0


	def create_selection(self, start, end=None):
		"""
		Creates selection from <code>start</code> to <code>end</code> character
		indexes. If <code>end</code> is ommited, this method should place caret
		and <code>start</code> index
		@type start: int
		@type end: int
		@example
		zen_editor.create_selection(10, 40)
		# move caret to 15th character
		zen_editor.create_selection(15)
		"""
		pass

	def get_current_line_range(self):
		"""
		Returns current line's start and end indexes
		@return: list of start and end indexes
		@example
		start, end = zen_editor.get_current_line_range();
		print('%s, %s' % (start, end))
		"""
		return 0, 0

	def get_caret_pos(self):
		""" Returns current caret position """
		return 0

	def set_caret_pos(self, pos):
		"""
		Set new caret position
		@type pos: int
		"""
		pass

	def get_current_line(self):
		"""
		Returns content of current line
		@return: str
		"""
		return ''

	def replace_content(self, value, start=None, end=None):
		"""
		Replace editor's content or it's part (from <code>start</code> to
		<code>end</code> index). If <code>value</code> contains
		<code>caret_placeholder</code>, the editor will put caret into
		this position. If you skip <code>start</code> and <code>end</code>
		arguments, the whole target's content will be replaced with
		<code>value</code>.

		If you pass <code>start</code> argument only,
		the <code>value</code> will be placed at <code>start</code> string
		index of current content.

		If you pass <code>start</code> and <code>end</code> arguments,
		the corresponding substring of current target's content will be
		replaced with <code>value</code>
		@param value: Content you want to paste
		@type value: str
		@param start: Start index of editor's content
		@type start: int
		@param end: End index of editor's content
		@type end: int
		"""
		pass

	def get_content(self):
		"""
		Returns editor's content
		@return: str
		"""
		return ''

	def get_syntax(self):
		"""
		Returns current editor's syntax mode
		@return: str
		"""
		return 'html'

	def get_profile_name(self):
		"""
		Returns current output profile name (@see zen_coding#setup_profile)
		@return {String}
		"""
		return 'xhtml'
	
	def prompt(self, title):
		"""
		Ask user to enter something
		@param title: Dialog title
		@type title: str
		@return: Entered data
		@since: 0.65
		"""
		return ''
	
	def get_selection(self):
		"""
		Returns current selection
		@return: str
		@since: 0.65
		"""
		return ''
	
	def get_file_path(self):
		"""
		Returns current editor's file path
		@return: str
		@since: 0.65 
		"""
		return ''
	


"""
Zen Coding settings
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
"""
zen_settings = {
			
#	Variables that can be placed inside snippets or abbreviations as ${variable}
#	${child} variable is reserved, don't use it
	'variables': {
		'lang': 'en',
		'locale': 'en-US',
		'charset': 'UTF-8',
		'profile': 'xhtml',
		
#		Inner element indentation
		'indentation': '\t',
		
		# newline variables, useful for wrapping
		'newline': '\n',
		'nl': '\n'
	},
	
	# common settings are used for quick injection of user-defined snippets
	'common': {
		
	},
	
	'css': {
		'extends': 'common',
		'filters': 'html,css',
		'snippets': {
			"@i": "@import url(|);",
			"@m": "@media print {\n\t|\n}",
			"@f": "@font-face {\n\tfont-family:|;\n\tsrc:url(|);\n}",
			"!": "!important",
			"pos": "position:|;",
			"pos:s": "position:static;",
			"pos:a": "position:absolute;",
			"pos:r": "position:relative;",
			"pos:f": "position:fixed;",
			"t": "top:|;",
			"t:a": "top:auto;",
			"r": "right:|;",
			"r:a": "right:auto;",
			"b": "bottom:|;",
			"b:a": "bottom:auto;",
			"brad": "-webkit-border-radius: ${1:radius};\n-moz-border-radius: $1;\n-ms-border-radius: $1;\nborder-radius: $1;",
			"bsha": "-webkit-box-shadow: ${1:hoff} ${2:voff} ${3:blur} ${4:rgba(0,0,0,0.5)};\n-moz-box-shadow: $1 $2 $3 $4;\n-ms-box-shadow: $1 $2 $3 $4;\nbox-shadow: $1 $2 $3 $4;",
			"l": "left:|;",
			"l:a": "left:auto;",
			"z": "z-index:|;",
			"z:a": "z-index:auto;",
			"fl": "float:|;",
			"fl:n": "float:none;",
			"fl:l": "float:left;",
			"fl:r": "float:right;",
			"cl": "clear:|;",
			"cl:n": "clear:none;",
			"cl:l": "clear:left;",
			"cl:r": "clear:right;",
			"cl:b": "clear:both;",
			"d": "display:|;",
			"d:n": "display:none;",
			"d:b": "display:block;",
			"d:i": "display:inline;",
			"d:ib": "display:inline-block;",
			"d:li": "display:list-item;",
			"d:ri": "display:run-in;",
			"d:cp": "display:compact;",
			"d:tb": "display:table;",
			"d:itb": "display:inline-table;",
			"d:tbcp": "display:table-caption;",
			"d:tbcl": "display:table-column;",
			"d:tbclg": "display:table-column-group;",
			"d:tbhg": "display:table-header-group;",
			"d:tbfg": "display:table-footer-group;",
			"d:tbr": "display:table-row;",
			"d:tbrg": "display:table-row-group;",
			"d:tbc": "display:table-cell;",
			"d:rb": "display:ruby;",
			"d:rbb": "display:ruby-base;",
			"d:rbbg": "display:ruby-base-group;",
			"d:rbt": "display:ruby-text;",
			"d:rbtg": "display:ruby-text-group;",
			"v": "visibility:|;",
			"v:v": "visibility:visible;",
			"v:h": "visibility:hidden;",
			"v:c": "visibility:collapse;",
			"ov": "overflow:|;",
			"ov:v": "overflow:visible;",
			"ov:h": "overflow:hidden;",
			"ov:s": "overflow:scroll;",
			"ov:a": "overflow:auto;",
			"ovx": "overflow-x:|;",
			"ovx:v": "overflow-x:visible;",
			"ovx:h": "overflow-x:hidden;",
			"ovx:s": "overflow-x:scroll;",
			"ovx:a": "overflow-x:auto;",
			"ovy": "overflow-y:|;",
			"ovy:v": "overflow-y:visible;",
			"ovy:h": "overflow-y:hidden;",
			"ovy:s": "overflow-y:scroll;",
			"ovy:a": "overflow-y:auto;",
			"ovs": "overflow-style:|;",
			"ovs:a": "overflow-style:auto;",
			"ovs:s": "overflow-style:scrollbar;",
			"ovs:p": "overflow-style:panner;",
			"ovs:m": "overflow-style:move;",
			"ovs:mq": "overflow-style:marquee;",
			"zoo": "zoom:1;",
			"cp": "clip:|;",
			"cp:a": "clip:auto;",
			"cp:r": "clip:rect(|);",
			"bxz": "box-sizing:|;",
			"bxz:cb": "box-sizing:content-box;",
			"bxz:bb": "box-sizing:border-box;",
			"bxsh": "box-shadow:|;",
			"bxsh:n": "box-shadow:none;",
			"bxsh:w": "-webkit-box-shadow:0 0 0 #000;",
			"bxsh:m": "-moz-box-shadow:0 0 0 0 #000;",
			"m": "margin:|;",
			"m:a": "margin:auto;",
			"m:0": "margin:0;",
			"m:2": "margin:0 0;",
			"m:3": "margin:0 0 0;",
			"m:4": "margin:0 0 0 0;",
			"mt": "margin-top:|;",
			"mt:a": "margin-top:auto;",
			"mr": "margin-right:|;",
			"mr:a": "margin-right:auto;",
			"mb": "margin-bottom:|;",
			"mb:a": "margin-bottom:auto;",
			"ml": "margin-left:|;",
			"ml:a": "margin-left:auto;",
			"p": "padding:|;",
			"p:0": "padding:0;",
			"p:2": "padding:0 0;",
			"p:3": "padding:0 0 0;",
			"p:4": "padding:0 0 0 0;",
			"pt": "padding-top:|;",
			"pr": "padding-right:|;",
			"pb": "padding-bottom:|;",
			"pl": "padding-left:|;",
			"w": "width:|;",
			"w:a": "width:auto;",
			"h": "height:|;",
			"h:a": "height:auto;",
			"maw": "max-width:|;",
			"maw:n": "max-width:none;",
			"mah": "max-height:|;",
			"mah:n": "max-height:none;",
			"miw": "min-width:|;",
			"mih": "min-height:|;",
			"o": "outline:|;",
			"o:n": "outline:none;",
			"oo": "outline-offset:|;",
			"ow": "outline-width:|;",
			"os": "outline-style:|;",
			"oc": "outline-color:#000;",
			"oc:i": "outline-color:invert;",
			"bd": "border:|;",
			"bd+": "border:1px solid #000;",
			"bd:n": "border:none;",
			"bdbk": "border-break:|;",
			"bdbk:c": "border-break:close;",
			"bdcl": "border-collapse:|;",
			"bdcl:c": "border-collapse:collapse;",
			"bdcl:s": "border-collapse:separate;",
			"bdc": "border-color:#000;",
			"bdi": "border-image:url(|);",
			"bdi:n": "border-image:none;",
			"bdi:w": "-webkit-border-image:url(|) 0 0 0 0 stretch stretch;",
			"bdi:m": "-moz-border-image:url(|) 0 0 0 0 stretch stretch;",
			"bdti": "border-top-image:url(|);",
			"bdti:n": "border-top-image:none;",
			"bdri": "border-right-image:url(|);",
			"bdri:n": "border-right-image:none;",
			"bdbi": "border-bottom-image:url(|);",
			"bdbi:n": "border-bottom-image:none;",
			"bdli": "border-left-image:url(|);",
			"bdli:n": "border-left-image:none;",
			"bdci": "border-corner-image:url(|);",
			"bdci:n": "border-corner-image:none;",
			"bdci:c": "border-corner-image:continue;",
			"bdtli": "border-top-left-image:url(|);",
			"bdtli:n": "border-top-left-image:none;",
			"bdtli:c": "border-top-left-image:continue;",
			"bdtri": "border-top-right-image:url(|);",
			"bdtri:n": "border-top-right-image:none;",
			"bdtri:c": "border-top-right-image:continue;",
			"bdbri": "border-bottom-right-image:url(|);",
			"bdbri:n": "border-bottom-right-image:none;",
			"bdbri:c": "border-bottom-right-image:continue;",
			"bdbli": "border-bottom-left-image:url(|);",
			"bdbli:n": "border-bottom-left-image:none;",
			"bdbli:c": "border-bottom-left-image:continue;",
			"bdf": "border-fit:|;",
			"bdf:c": "border-fit:clip;",
			"bdf:r": "border-fit:repeat;",
			"bdf:sc": "border-fit:scale;",
			"bdf:st": "border-fit:stretch;",
			"bdf:ow": "border-fit:overwrite;",
			"bdf:of": "border-fit:overflow;",
			"bdf:sp": "border-fit:space;",
			"bdl": "border-length:|;",
			"bdl:a": "border-length:auto;",
			"bdsp": "border-spacing:|;",
			"bds": "border-style:|;",
			"bds:n": "border-style:none;",
			"bds:h": "border-style:hidden;",
			"bds:dt": "border-style:dotted;",
			"bds:ds": "border-style:dashed;",
			"bds:s": "border-style:solid;",
			"bds:db": "border-style:double;",
			"bds:dtds": "border-style:dot-dash;",
			"bds:dtdtds": "border-style:dot-dot-dash;",
			"bds:w": "border-style:wave;",
			"bds:g": "border-style:groove;",
			"bds:r": "border-style:ridge;",
			"bds:i": "border-style:inset;",
			"bds:o": "border-style:outset;",
			"bdw": "border-width:|;",
			"bdt": "border-top:|;",
			"bdt+": "border-top:1px solid #000;",
			"bdt:n": "border-top:none;",
			"bdtw": "border-top-width:|;",
			"bdts": "border-top-style:|;",
			"bdts:n": "border-top-style:none;",
			"bdtc": "border-top-color:#000;",
			"bdr": "border-right:|;",
			"bdr+": "border-right:1px solid #000;",
			"bdr:n": "border-right:none;",
			"bdrw": "border-right-width:|;",
			"bdrs": "border-right-style:|;",
			"bdrs:n": "border-right-style:none;",
			"bdrc": "border-right-color:#000;",
			"bdb": "border-bottom:|;",
			"bdb+": "border-bottom:1px solid #000;",
			"bdb:n": "border-bottom:none;",
			"bdbw": "border-bottom-width:|;",
			"bdbs": "border-bottom-style:|;",
			"bdbs:n": "border-bottom-style:none;",
			"bdbc": "border-bottom-color:#000;",
			"bdl": "border-left:|;",
			"bdl+": "border-left:1px solid #000;",
			"bdl:n": "border-left:none;",
			"bdlw": "border-left-width:|;",
			"bdls": "border-left-style:|;",
			"bdls:n": "border-left-style:none;",
			"bdlc": "border-left-color:#000;",
			"bdrs": "border-radius:|;",
			"bdtrrs": "border-top-right-radius:|;",
			"bdtlrs": "border-top-left-radius:|;",
			"bdbrrs": "border-bottom-right-radius:|;",
			"bdblrs": "border-bottom-left-radius:|;",
			"bg": "background:|;",
			"bg+": "background:#FFF url(|) 0 0 no-repeat;",
			"bg:n": "background:none;",
			"bg:ie": "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='${1:x}.png',sizingMethod='${2:crop}');",
			"bgc": "background-color:#FFF;",
			"bgi": "background-image:url(|);",
			"bgi:n": "background-image:none;",
			"bgr": "background-repeat:|;",
			"bgr:n": "background-repeat:no-repeat;",
			"bgr:x": "background-repeat:repeat-x;",
			"bgr:y": "background-repeat:repeat-y;",
			"bga": "background-attachment:|;",
			"bga:f": "background-attachment:fixed;",
			"bga:s": "background-attachment:scroll;",
			"bgp": "background-position:0 0;",
			"bgpx": "background-position-x:|;",
			"bgpy": "background-position-y:|;",
			"bgbk": "background-break:|;",
			"bgbk:bb": "background-break:bounding-box;",
			"bgbk:eb": "background-break:each-box;",
			"bgbk:c": "background-break:continuous;",
			"bgcp": "background-clip:|;",
			"bgcp:bb": "background-clip:border-box;",
			"bgcp:pb": "background-clip:padding-box;",
			"bgcp:cb": "background-clip:content-box;",
			"bgcp:nc": "background-clip:no-clip;",
			"bgo": "background-origin:|;",
			"bgo:pb": "background-origin:padding-box;",
			"bgo:bb": "background-origin:border-box;",
			"bgo:cb": "background-origin:content-box;",
			"bgz": "background-size:|;",
			"bgz:a": "background-size:auto;",
			"bgz:ct": "background-size:contain;",
			"bgz:cv": "background-size:cover;",
			"c": "color:#000;",
			"tbl": "table-layout:|;",
			"tbl:a": "table-layout:auto;",
			"tbl:f": "table-layout:fixed;",
			"cps": "caption-side:|;",
			"cps:t": "caption-side:top;",
			"cps:b": "caption-side:bottom;",
			"ec": "empty-cells:|;",
			"ec:s": "empty-cells:show;",
			"ec:h": "empty-cells:hide;",
			"lis": "list-style:|;",
			"lis:n": "list-style:none;",
			"lisp": "list-style-position:|;",
			"lisp:i": "list-style-position:inside;",
			"lisp:o": "list-style-position:outside;",
			"list": "list-style-type:|;",
			"list:n": "list-style-type:none;",
			"list:d": "list-style-type:disc;",
			"list:c": "list-style-type:circle;",
			"list:s": "list-style-type:square;",
			"list:dc": "list-style-type:decimal;",
			"list:dclz": "list-style-type:decimal-leading-zero;",
			"list:lr": "list-style-type:lower-roman;",
			"list:ur": "list-style-type:upper-roman;",
			"lisi": "list-style-image:|;",
			"lisi:n": "list-style-image:none;",
			"q": "quotes:|;",
			"q:n": "quotes:none;",
			"q:ru": "quotes:'\00AB' '\00BB' '\201E' '\201C';",
			"q:en": "quotes:'\201C' '\201D' '\2018' '\2019';",
			"ct": "content:|;",
			"ct:n": "content:normal;",
			"ct:oq": "content:open-quote;",
			"ct:noq": "content:no-open-quote;",
			"ct:cq": "content:close-quote;",
			"ct:ncq": "content:no-close-quote;",
			"ct:a": "content:attr(|);",
			"ct:c": "content:counter(|);",
			"ct:cs": "content:counters(|);",
			"coi": "counter-increment:|;",
			"cor": "counter-reset:|;",
			"va": "vertical-align:|;",
			"va:sup": "vertical-align:super;",
			"va:t": "vertical-align:top;",
			"va:tt": "vertical-align:text-top;",
			"va:m": "vertical-align:middle;",
			"va:bl": "vertical-align:baseline;",
			"va:b": "vertical-align:bottom;",
			"va:tb": "vertical-align:text-bottom;",
			"va:sub": "vertical-align:sub;",
			"ta": "text-align:|;",
			"ta:l": "text-align:left;",
			"ta:c": "text-align:center;",
			"ta:r": "text-align:right;",
			"tal": "text-align-last:|;",
			"tal:a": "text-align-last:auto;",
			"tal:l": "text-align-last:left;",
			"tal:c": "text-align-last:center;",
			"tal:r": "text-align-last:right;",
			"td": "text-decoration:|;",
			"td:n": "text-decoration:none;",
			"td:u": "text-decoration:underline;",
			"td:o": "text-decoration:overline;",
			"td:l": "text-decoration:line-through;",
			"te": "text-emphasis:|;",
			"te:n": "text-emphasis:none;",
			"te:ac": "text-emphasis:accent;",
			"te:dt": "text-emphasis:dot;",
			"te:c": "text-emphasis:circle;",
			"te:ds": "text-emphasis:disc;",
			"te:b": "text-emphasis:before;",
			"te:a": "text-emphasis:after;",
			"th": "text-height:|;",
			"th:a": "text-height:auto;",
			"th:f": "text-height:font-size;",
			"th:t": "text-height:text-size;",
			"th:m": "text-height:max-size;",
			"ti": "text-indent:|;",
			"ti:-": "text-indent:-9999px;",
			"tj": "text-justify:|;",
			"tj:a": "text-justify:auto;",
			"tj:iw": "text-justify:inter-word;",
			"tj:ii": "text-justify:inter-ideograph;",
			"tj:ic": "text-justify:inter-cluster;",
			"tj:d": "text-justify:distribute;",
			"tj:k": "text-justify:kashida;",
			"tj:t": "text-justify:tibetan;",
			"to": "text-outline:|;",
			"to+": "text-outline:0 0 #000;",
			"to:n": "text-outline:none;",
			"tr": "text-replace:|;",
			"tr:n": "text-replace:none;",
			"tt": "text-transform:|;",
			"tt:n": "text-transform:none;",
			"tt:c": "text-transform:capitalize;",
			"tt:u": "text-transform:uppercase;",
			"tt:l": "text-transform:lowercase;",
			"tw": "text-wrap:|;",
			"tw:n": "text-wrap:normal;",
			"tw:no": "text-wrap:none;",
			"tw:u": "text-wrap:unrestricted;",
			"tw:s": "text-wrap:suppress;",
			"tsh": "text-shadow:|;",
			"tsh+": "text-shadow:0 0 0 #000;",
			"tsh:n": "text-shadow:none;",
			"lh": "line-height:|;",
			"whs": "white-space:|;",
			"whs:n": "white-space:normal;",
			"whs:p": "white-space:pre;",
			"whs:nw": "white-space:nowrap;",
			"whs:pw": "white-space:pre-wrap;",
			"whs:pl": "white-space:pre-line;",
			"whsc": "white-space-collapse:|;",
			"whsc:n": "white-space-collapse:normal;",
			"whsc:k": "white-space-collapse:keep-all;",
			"whsc:l": "white-space-collapse:loose;",
			"whsc:bs": "white-space-collapse:break-strict;",
			"whsc:ba": "white-space-collapse:break-all;",
			"wob": "word-break:|;",
			"wob:n": "word-break:normal;",
			"wob:k": "word-break:keep-all;",
			"wob:l": "word-break:loose;",
			"wob:bs": "word-break:break-strict;",
			"wob:ba": "word-break:break-all;",
			"wos": "word-spacing:|;",
			"wow": "word-wrap:|;",
			"wow:nm": "word-wrap:normal;",
			"wow:n": "word-wrap:none;",
			"wow:u": "word-wrap:unrestricted;",
			"wow:s": "word-wrap:suppress;",
			"lts": "letter-spacing:|;",
			"f": "font:|;",
			"f+": "font:1em Arial,sans-serif;",
			"fw": "font-weight:|;",
			"fw:n": "font-weight:normal;",
			"fw:b": "font-weight:bold;",
			"fw:br": "font-weight:bolder;",
			"fw:lr": "font-weight:lighter;",
			"fs": "font-style:|;",
			"fs:n": "font-style:normal;",
			"fs:i": "font-style:italic;",
			"fs:o": "font-style:oblique;",
			"fv": "font-variant:|;",
			"fv:n": "font-variant:normal;",
			"fv:sc": "font-variant:small-caps;",
			"fz": "font-size:|;",
			"fza": "font-size-adjust:|;",
			"fza:n": "font-size-adjust:none;",
			"ff": "font-family:|;",
			"ff:s": "font-family:serif;",
			"ff:ss": "font-family:sans-serif;",
			"ff:c": "font-family:cursive;",
			"ff:f": "font-family:fantasy;",
			"ff:m": "font-family:monospace;",
			"fef": "font-effect:|;",
			"fef:n": "font-effect:none;",
			"fef:eg": "font-effect:engrave;",
			"fef:eb": "font-effect:emboss;",
			"fef:o": "font-effect:outline;",
			"fem": "font-emphasize:|;",
			"femp": "font-emphasize-position:|;",
			"femp:b": "font-emphasize-position:before;",
			"femp:a": "font-emphasize-position:after;",
			"fems": "font-emphasize-style:|;",
			"fems:n": "font-emphasize-style:none;",
			"fems:ac": "font-emphasize-style:accent;",
			"fems:dt": "font-emphasize-style:dot;",
			"fems:c": "font-emphasize-style:circle;",
			"fems:ds": "font-emphasize-style:disc;",
			"fsm": "font-smooth:|;",
			"fsm:a": "font-smooth:auto;",
			"fsm:n": "font-smooth:never;",
			"fsm:aw": "font-smooth:always;",
			"fst": "font-stretch:|;",
			"fst:n": "font-stretch:normal;",
			"fst:uc": "font-stretch:ultra-condensed;",
			"fst:ec": "font-stretch:extra-condensed;",
			"fst:c": "font-stretch:condensed;",
			"fst:sc": "font-stretch:semi-condensed;",
			"fst:se": "font-stretch:semi-expanded;",
			"fst:e": "font-stretch:expanded;",
			"fst:ee": "font-stretch:extra-expanded;",
			"fst:ue": "font-stretch:ultra-expanded;",
			"op": "opacity:|;",
			"op:ie": "filter:progid:DXImageTransform.Microsoft.Alpha(Opacity=100);",
			"op:ms": "-ms-filter:'progid:DXImageTransform.Microsoft.Alpha(Opacity=100)';",
			"rz": "resize:|;",
			"rz:n": "resize:none;",
			"rz:b": "resize:both;",
			"rz:h": "resize:horizontal;",
			"rz:v": "resize:vertical;",
			"cur": "cursor:|;",
			"cur:a": "cursor:auto;",
			"cur:d": "cursor:default;",
			"cur:c": "cursor:crosshair;",
			"cur:ha": "cursor:hand;",
			"cur:he": "cursor:help;",
			"cur:m": "cursor:move;",
			"cur:p": "cursor:pointer;",
			"cur:t": "cursor:text;",
			"pgbb": "page-break-before:|;",
			"pgbb:au": "page-break-before:auto;",
			"pgbb:al": "page-break-before:always;",
			"pgbb:l": "page-break-before:left;",
			"pgbb:r": "page-break-before:right;",
			"pgbi": "page-break-inside:|;",
			"pgbi:au": "page-break-inside:auto;",
			"pgbi:av": "page-break-inside:avoid;",
			"pgba": "page-break-after:|;",
			"pgba:au": "page-break-after:auto;",
			"pgba:al": "page-break-after:always;",
			"pgba:l": "page-break-after:left;",
			"pgba:r": "page-break-after:right;",
			"orp": "orphans:|;",
			"wid": "widows:|;"
		}
	},
	
	'html': {
		'extends': 'common',
		'filters': 'html',
		'snippets': {
			'cc:ie6': '<!--[if lte IE 6]>\n\t${child}|\n<![endif]-->',
			'cc:ie': '<!--[if IE]>\n\t${child}|\n<![endif]-->',
			'cc:noie': '<!--[if !IE]><!-->\n\t${child}|\n<!--<![endif]-->',
			'html:4t': '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' +
					'<html lang="${lang}">\n' +
					'<head>\n' +
					'	<meta http-equiv="Content-Type" content="text/html;charset=${charset}">\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>',
			
			'html:4s': '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">\n' +
					'<html lang="${lang}">\n' +
					'<head>\n' +
					'	<meta http-equiv="Content-Type" content="text/html;charset=${charset}">\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>',
			
			'html:xt': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n' +
					'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="${lang}">\n' +
					'<head>\n' +
					'	<meta http-equiv="Content-Type" content="text/html;charset=${charset}" />\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>',
			
			'html:xs': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n' +
					'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="${lang}">\n' +
					'<head>\n' +
					'	<meta http-equiv="Content-Type" content="text/html;charset=${charset}" />\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>',
			
			'html:xxs': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n' +
					'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="${lang}">\n' +
					'<head>\n' +
					'	<meta http-equiv="Content-Type" content="text/html;charset=${charset}" />\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>',
			
			'html:5': '<!DOCTYPE HTML>\n' +
					'<html lang="${locale}">\n' +
					'<head>\n' +
					'	<meta charset="${charset}">\n' +
					'	<title></title>\n' +
					'</head>\n' +
					'<body>\n\t${child}|\n</body>\n' +
					'</html>'
		},
		
		'abbreviations': {
			'a': '<a href=""></a>',
			'a:link': '<a href="http://|"></a>',
			'a:mail': '<a href="mailto:|"></a>',
			'abbr': '<abbr title=""></abbr>',
			'acronym': '<acronym title=""></acronym>',
			'base': '<base href="" />',
			'bdo': '<bdo dir=""></bdo>',
			'bdo:r': '<bdo dir="rtl"></bdo>',
			'bdo:l': '<bdo dir="ltr"></bdo>',
			'link:css': '<link rel="stylesheet" type="text/css" href="${1:style}.css" media="all" />',
			'link:print': '<link rel="stylesheet" type="text/css" href="|print.css" media="print" />',
			'link:favicon': '<link rel="shortcut icon" type="image/x-icon" href="|favicon.ico" />',
			'link:touch': '<link rel="apple-touch-icon" href="|favicon.png" />',
			'link:rss': '<link rel="alternate" type="application/rss+xml" title="RSS" href="|rss.xml" />',
			'link:atom': '<link rel="alternate" type="application/atom+xml" title="Atom" href="atom.xml" />',
			'meta:utf': '<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />',
			'meta:win': '<meta http-equiv="Content-Type" content="text/html;charset=Win-1251" />',
			'meta:compat': '<meta http-equiv="X-UA-Compatible" content="IE=7" />',
			'style': '<style type="text/css"></style>',
			'script': '<script type="text/javascript"></script>',
			'script:src': '<script type="text/javascript" src=""></script>',
			'img': '<img src="" alt="" />',
			'iframe': '<iframe src="" frameborder="0"></iframe>',
			'embed': '<embed src="" type="" />',
			'object': '<object data="" type=""></object>',
			'param': '<param name="" value="" />',
			'map': '<map name=""></map>',
			'area': '<area shape="" coords="" href="" alt="" />',
			'area:d': '<area shape="default" href="" alt="" />',
			'area:c': '<area shape="circle" coords="" href="" alt="" />',
			'area:r': '<area shape="rect" coords="" href="" alt="" />',
			'area:p': '<area shape="poly" coords="" href="" alt="" />',
			'link': '<link rel="stylesheet" href="" />',
			'form': '<form action=""></form>',
			'form:get': '<form action="" method="get"></form>',
			'form:post': '<form action="" method="post"></form>',
			'label': '<label for=""></label>',
			'input': '<input type="" />',
			'input:hidden': '<input type="hidden" name="" />',
			'input:h': '<input type="hidden" name="" />',
			'input:text': '<input type="text" name="" id="" />',
			'input:t': '<input type="text" name="" id="" />',
			'input:search': '<input type="search" name="" id="" />',
			'input:email': '<input type="email" name="" id="" />',
			'input:url': '<input type="url" name="" id="" />',
			'input:password': '<input type="password" name="" id="" />',
			'input:p': '<input type="password" name="" id="" />',
			'input:datetime': '<input type="datetime" name="" id="" />',
			'input:date': '<input type="date" name="" id="" />',
			'input:datetime-local': '<input type="datetime-local" name="" id="" />',
			'input:month': '<input type="month" name="" id="" />',
			'input:week': '<input type="week" name="" id="" />',
			'input:time': '<input type="time" name="" id="" />',
			'input:number': '<input type="number" name="" id="" />',
			'input:color': '<input type="color" name="" id="" />',
			'input:checkbox': '<input type="checkbox" name="" id="" />',
			'input:c': '<input type="checkbox" name="" id="" />',
			'input:radio': '<input type="radio" name="" id="" />',
			'input:r': '<input type="radio" name="" id="" />',
			'input:range': '<input type="range" name="" id="" />',
			'input:file': '<input type="file" name="" id="" />',
			'input:f': '<input type="file" name="" id="" />',
			'input:submit': '<input type="submit" value="" />',
			'input:s': '<input type="submit" value="" />',
			'input:image': '<input type="image" src="" alt="" />',
			'input:i': '<input type="image" src="" alt="" />',
			'input:reset': '<input type="reset" value="" />',
			'input:button': '<input type="button" value="" />',
			'input:b': '<input type="button" value="" />',
			'select': '<select name="" id=""></select>',
			'option': '<option value=""></option>',
			'textarea': '<textarea name="" id="" cols="30" rows="10"></textarea>',
			'menu:context': '<menu type="context"></menu>',
			'menu:c': '<menu type="context"></menu>',
			'menu:toolbar': '<menu type="toolbar"></menu>',
			'menu:t': '<menu type="toolbar"></menu>',
			'video': '<video src=""></video>',
			'audio': '<audio src=""></audio>',
			'html:xml': '<html xmlns="http://www.w3.org/1999/xhtml"></html>',
			'bq': '<blockquote></blockquote>',
			'acr': '<acronym></acronym>',
			'fig': '<figure></figure>',
			'ifr': '<iframe></iframe>',
			'emb': '<embed></embed>',
			'obj': '<object></object>',
			'src': '<source></source>',
			'cap': '<caption></caption>',
			'colg': '<colgroup></colgroup>',
			'fst': '<fieldset></fieldset>',
			'btn': '<button></button>',
			'optg': '<optgroup></optgroup>',
			'opt': '<option></option>',
			'tarea': '<textarea></textarea>',
			'leg': '<legend></legend>',
			'sect': '<section></section>',
			'art': '<article></article>',
			'hdr': '<header></header>',
			'ftr': '<footer></footer>',
			'adr': '<address></address>',
			'dlg': '<dialog></dialog>',
			'str': '<strong></strong>',
			'prog': '<progress></progress>',
			'fset': '<fieldset></fieldset>',
			'datag': '<datagrid></datagrid>',
			'datal': '<datalist></datalist>',
			'kg': '<keygen></keygen>',
			'out': '<output></output>',
			'det': '<details></details>',
			'cmd': '<command></command>',
			
#			expandos
			'ol+': 'ol>li',
			'ul+': 'ul>li',
			'dl+': 'dl>dt+dd',
			'map+': 'map>area',
			'table+': 'table>tr>td',
			'colgroup+': 'colgroup>col',
			'colg+': 'colgroup>col',
			'tr+': 'tr>td',
			'select+': 'select>option',
			'optgroup+': 'optgroup>option',
			'optg+': 'optgroup>option'

		},
		
		'element_types': {
			'empty': 'area,base,basefont,br,col,frame,hr,img,input,isindex,link,meta,param,embed,keygen,command',
			'block_level': 'address,applet,blockquote,button,center,dd,del,dir,div,dl,dt,fieldset,form,frameset,hr,iframe,ins,isindex,li,link,map,menu,noframes,noscript,object,ol,p,pre,script,table,tbody,td,tfoot,th,thead,tr,ul,h1,h2,h3,h4,h5,h6',
			'inline_level': 'a,abbr,acronym,applet,b,basefont,bdo,big,br,button,cite,code,del,dfn,em,font,i,iframe,img,input,ins,kbd,label,map,object,q,s,samp,script,select,small,span,strike,strong,sub,sup,textarea,tt,u,var'
		}
	},
	
	'xml': {
		'extends': 'html',
		'filters': 'html'
	},
	
	'xsl': {
		'extends': 'common,html',
		'filters': 'html, xsl',
		'abbreviations': {
			'tm': '<xsl:template match="" mode=""></xsl:template>',
			'tmatch': 'tm',
			'tn': '<xsl:template name=""></xsl:template>',
			'tname': 'tn',
			'xsl:when': '<xsl:when test=""></xsl:when>',
			'wh': 'xsl:when',
			'var': '<xsl:variable name="">|</xsl:variable>',
			'vare': '<xsl:variable name="" select=""/>',
			'if': '<xsl:if test=""></xsl:if>',
			'call': '<xsl:call-template name=""/>',
			'attr': '<xsl:attribute name=""></xsl:attribute>',
			'wp': '<xsl:with-param name="" select=""/>',
			'par': '<xsl:param name="" select=""/>',
			'val': '<xsl:value-of select=""/>',
			'co': '<xsl:copy-of select=""/>',
			'each': '<xsl:for-each select=""></xsl:for-each>',
			'ap': '<xsl:apply-templates select="" mode=""/>',
			
#			expandos
			'choose+': 'xsl:choose>xsl:when+xsl:otherwise'
		}
	},
	
	'haml': {
		'filters': 'haml',
		'extends': 'html'
	}
}


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Context-independent xHTML pair matcher
Use method <code>match(html, start_ix)</code> to find matching pair.
If pair was found, this function returns a list of indexes where tag pair 
starts and ends. If pair wasn't found, <code>None</code> will be returned.

The last matched (or unmatched) result is saved in <code>last_match</code> 
dictionary for later use.

@author: Sergey Chikuyonok (serge.che@gmail.com)
'''
import re

start_tag = r'<([\w\:\-]+)((?:\s+[\w\-:]+(?:\s*=\s*(?:(?:"[^"]*")|(?:\'[^\']*\')|[^>\s]+))?)*)\s*(\/?)>'
end_tag = r'<\/([\w\:\-]+)[^>]*>'
attr = r'([\w\-:]+)(?:\s*=\s*(?:(?:"((?:\\.|[^"])*)")|(?:\'((?:\\.|[^\'])*)\')|([^>\s]+)))?'

"Last matched HTML pair"
last_match = {
	'opening_tag': None, # Tag() or Comment() object
	'closing_tag': None, # Tag() or Comment() object
	'start_ix': -1,
	'end_ix': -1
}

cur_mode = 'xhtml'
"Current matching mode"

def set_mode(new_mode):
	global cur_mode
	if new_mode != 'html': new_mode = 'xhtml'
	cur_mode = new_mode

def make_map(elems):
	"""
	Create dictionary of elements for faster searching
	@param elems: Elements, separated by comma
	@type elems: str
	"""
	obj = {}
	for elem in elems.split(','):
			obj[elem] = True

	return obj

# Empty Elements - HTML 4.01
empty = make_map("area,base,basefont,br,col,frame,hr,img,input,isindex,link,meta,param,embed");

# Block Elements - HTML 4.01
block = make_map("address,applet,blockquote,button,center,dd,dir,div,dl,dt,fieldset,form,frameset,hr,iframe,isindex,li,map,menu,noframes,noscript,object,ol,p,pre,script,table,tbody,td,tfoot,th,thead,tr,ul");

# Inline Elements - HTML 4.01
inline = make_map("a,abbr,acronym,applet,b,basefont,bdo,big,br,button,cite,code,del,dfn,em,font,i,iframe,img,input,ins,kbd,label,map,object,q,s,samp,select,small,span,strike,strong,sub,sup,textarea,tt,u,var");

# Elements that you can, intentionally, leave open
# (and which close themselves)
close_self = make_map("colgroup,dd,dt,li,options,p,td,tfoot,th,thead,tr");

# Attributes that have their values filled in disabled="disabled"
fill_attrs = make_map("checked,compact,declare,defer,disabled,ismap,multiple,nohref,noresize,noshade,nowrap,readonly,selected");

#Special Elements (can contain anything)
# serge.che: parsing data inside <scipt> elements is a "feature"
special = make_map("style");

class Tag():
	"""Matched tag"""
	def __init__(self, match, ix):
		"""
		@type match: MatchObject
		@param match: Matched HTML tag
		@type ix: int
		@param ix: Tag's position
		"""
		global cur_mode
		
		name = match.group(1).lower()
		self.name = name
		self.full_tag = match.group(0)
		self.start = ix
		self.end = ix + len(self.full_tag)
		self.unary = ( len(match.groups()) > 2 and bool(match.group(3)) ) or (name in empty and cur_mode == 'html')
		self.type = 'tag'
		self.close_self = (name in close_self and cur_mode == 'html')

class Comment():
	"Matched comment"
	def __init__(self, start, end):
		self.start = start
		self.end = end
		self.type = 'comment'

def make_range(opening_tag=None, closing_tag=None, ix=0):
	"""
	Makes selection ranges for matched tag pair
	@type opening_tag: Tag
    @type closing_tag: Tag
    @type ix: int
    @return list
	"""
	start_ix, end_ix = -1, -1
	
	if opening_tag and not closing_tag: # unary element
		start_ix = opening_tag.start
		end_ix = opening_tag.end
	elif opening_tag and closing_tag: # complete element
		if (opening_tag.start < ix and opening_tag.end > ix) or (closing_tag.start <= ix and closing_tag.end > ix):
			start_ix = opening_tag.start
			end_ix = closing_tag.end;
		else:
			start_ix = opening_tag.end
			end_ix = closing_tag.start
	
	return start_ix, end_ix

def save_match(opening_tag=None, closing_tag=None, ix=0):
	"""
	Save matched tag for later use and return found indexes
    @type opening_tag: Tag
    @type closing_tag: Tag
    @type ix: int
    @return list
	"""
	last_match['opening_tag'] = opening_tag; 
	last_match['closing_tag'] = closing_tag;
	
	last_match['start_ix'], last_match['end_ix'] = make_range(opening_tag, closing_tag, ix)
	
	return last_match['start_ix'] != -1 and (last_match['start_ix'], last_match['end_ix']) or (None, None)

def match(html, start_ix, mode='xhtml'):
	"""
	Search for matching tags in <code>html</code>, starting from
	<code>start_ix</code> position. The result is automatically saved
	in <code>last_match</code> property
	"""
	return _find_pair(html, start_ix, mode, save_match)

def find(html, start_ix, mode='xhtml'):
	"""
	Search for matching tags in <code>html</code>, starting from
	<code>start_ix</code> position.
	"""
	return _find_pair(html, start_ix, mode)

def get_tags(html, start_ix, mode='xhtml'):
	"""
	Search for matching tags in <code>html</code>, starting from 
	<code>start_ix</code> position. The difference between 
	<code>match</code> function itself is that <code>get_tags</code> 
	method doesn't save matched result in <code>last_match</code> property 
	and returns array of opening and closing tags
	This method is generally used for lookups
	"""
	return _find_pair(html, start_ix, mode, lambda op, cl=None, ix=0: (op, cl) if op and op.type == 'tag' else None)


def _find_pair(html, start_ix, mode='xhtml', action=make_range):
	"""
	Search for matching tags in <code>html</code>, starting from
	<code>start_ix</code> position
	
	@param html: Code to search
	@type html: str
	
	@param start_ix: Character index where to start searching pair
	(commonly, current caret position)
	@type start_ix: int
	
	@param action: Function that creates selection range
	@type action: function
	
	@return: list
	"""

	forward_stack = []
	backward_stack = []
	opening_tag = None
	closing_tag = None
	html_len = len(html)
	
	set_mode(mode)

	def has_match(substr, start=None):
		if start is None:
			start = ix

		return html.find(substr, start) == start


	def find_comment_start(start_pos):
		while start_pos:
			if html[start_pos] == '<' and has_match('<!--', start_pos):
				break

			start_pos -= 1

		return start_pos

#    find opening tag
	ix = start_ix - 1
	while ix >= 0:
		ch = html[ix]
		if ch == '<':
			check_str = html[ix:]
			m = re.match(end_tag, check_str)
			if m:  # found closing tag
				tmp_tag = Tag(m, ix)
				if tmp_tag.start < start_ix and tmp_tag.end > start_ix: # direct hit on searched closing tag
					closing_tag = tmp_tag
				else:
					backward_stack.append(tmp_tag)
			else:
				m = re.match(start_tag, check_str)
				if m: # found opening tag
					tmp_tag = Tag(m, ix);
					if tmp_tag.unary:
						if tmp_tag.start < start_ix and tmp_tag.end > start_ix: # exact match
							return action(tmp_tag, None, start_ix)
					elif backward_stack and backward_stack[-1].name == tmp_tag.name:
						backward_stack.pop()
					else: # found nearest unclosed tag
						opening_tag = tmp_tag
						break
				elif check_str.startswith('<!--'): # found comment start
					end_ix = check_str.find('-->') + ix + 3;
					if ix < start_ix and end_ix >= start_ix:
						return action(Comment(ix, end_ix))
		elif ch == '-' and has_match('-->'): # found comment end
			# search left until comment start is reached
			ix = find_comment_start(ix)

		ix -= 1
		
	if not opening_tag:
		return action(None)
	
	# find closing tag
	if not closing_tag:
		ix = start_ix
		while ix < html_len:
			ch = html[ix]
			if ch == '<':
				check_str = html[ix:]
				m = re.match(start_tag, check_str)
				if m: # found opening tag
					tmp_tag = Tag(m, ix);
					if not tmp_tag.unary:
						forward_stack.append(tmp_tag)
				else:
					m = re.match(end_tag, check_str)
					if m:   #found closing tag
						tmp_tag = Tag(m, ix);
						if forward_stack and forward_stack[-1].name == tmp_tag.name:
							forward_stack.pop()
						else:  # found matched closing tag
							closing_tag = tmp_tag;
							break
					elif has_match('<!--'): # found comment
						ix += check_str.find('-->') + 2
						continue
			elif ch == '-' and has_match('-->'):
				# looks like cursor was inside comment with invalid HTML
				if not forward_stack or forward_stack[-1].type != 'comment':
					end_ix = ix + 3
					return action(Comment( find_comment_start(ix), end_ix ))
				
			ix += 1
	
	return action(opening_tag, closing_tag, start_ix)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Format CSS properties: add space after property name:
padding:0; -> padding: 0;
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re
import zencoding

re_css_prop = re.compile(r'([\w\-]+\s*:)(?!:)\s*')

@zencoding.filter('fc')
def process(tree, profile):
	for item in tree.children:
		# CSS properties are always snippets 
		if item.type == 'snippet':
			item.start = re_css_prop.sub(r'\1 ', item.start)
		
		process(item, profile)
		
	return tree


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path
import sys

# import all filters
__sub_modules = []

for file in os.listdir(os.path.dirname(__file__)):
	name, ext = os.path.splitext(file)
	if ext.lower() == '.py':
		__sub_modules.append(name)
		
__import__(__name__, globals(), locals(), __sub_modules)

del __sub_modules


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Trim filter: removes characters at the beginning of the text
content that indicates lists: numbers, #, *, -, etc.
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re
import zencoding

re_indicators = re.compile(r'^([\s|\u00a0])?[\d|#|\-|\*|\u2022]+\.?\s*')

@zencoding.filter('t')
def process(tree, profile):
	for item in tree.children:
		if item.content:
			item.content = re_indicators.sub('', item.content)
		
		process(item, profile)
	
	return tree


#!/usr/bin/env python

# This code is original from jsmin by Douglas Crockford, it was translated to
# Python by Baruch Even. It was rewritten by Dave St.Germain for speed.
#
# The MIT License (MIT)
#
# Copyright (c) 2013 Dave St.Germain
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import sys
is_3 = sys.version_info >= (3, 0)
if is_3:
    import io
else:
    import StringIO
    try:
        import cStringIO
    except ImportError:
        cStringIO = None


__all__ = ['jsmin', 'JavascriptMinify']
__version__ = '2.1.6'


def jsmin(js, **kwargs):
    """
    returns a minified version of the javascript string
    """
    if not is_3:
        if cStringIO and not isinstance(js, unicode):
            # strings can use cStringIO for a 3x performance
            # improvement, but unicode (in python2) cannot
            klass = cStringIO.StringIO
        else:
            klass = StringIO.StringIO
    else:
        klass = io.StringIO
    ins = klass(js)
    outs = klass()
    JavascriptMinify(ins, outs, **kwargs).minify()
    return outs.getvalue()


class JavascriptMinify(object):
    """
    Minify an input stream of javascript, writing
    to an output stream
    """

    def __init__(self, instream=None, outstream=None, quote_chars="'\""):
        self.ins = instream
        self.outs = outstream
        self.quote_chars = quote_chars

    def minify(self, instream=None, outstream=None):
        if instream and outstream:
            self.ins, self.outs = instream, outstream

        self.is_return = False
        self.return_buf = ''

        def write(char):
            # all of this is to support literal regular expressions.
            # sigh
            if char in 'return':
                self.return_buf += char
                self.is_return = self.return_buf == 'return'
            else:
                self.return_buf = ''
            self.outs.write(char)
            if self.is_return:
                self.return_buf = ''

        read = self.ins.read

        space_strings = "abcdefghijklmnopqrstuvwxyz"\
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_$\\"
        self.space_strings = space_strings
        starters, enders = '{[(+-', '}])+-/' + self.quote_chars
        newlinestart_strings = starters + space_strings + self.quote_chars
        newlineend_strings = enders + space_strings + self.quote_chars
        self.newlinestart_strings = newlinestart_strings
        self.newlineend_strings = newlineend_strings

        do_newline = False
        do_space = False
        escape_slash_count = 0
        in_quote = ''
        quote_buf = []

        previous = ';'
        previous_non_space = ';'
        next1 = read(1)

        while next1:
            next2 = read(1)
            if in_quote:
                quote_buf.append(next1)

                if next1 == in_quote:
                    numslashes = 0
                    for c in reversed(quote_buf[:-1]):
                        if c != '\\':
                            break
                        else:
                            numslashes += 1
                    if numslashes % 2 == 0:
                        in_quote = ''
                        write(''.join(quote_buf))
            elif next1 in '\r\n':
                next2, do_newline = self.newline(
                    previous_non_space, next2, do_newline)
            elif next1 < '!':
                if (previous_non_space in space_strings \
                    or previous_non_space > '~') \
                    and (next2 in space_strings or next2 > '~'):
                    do_space = True
                elif previous_non_space in '-+' and next2 == previous_non_space:
                    # protect against + ++ or - -- sequences
                    do_space = True
                elif self.is_return and next2 == '/':
                    # returning a regex...
                    write(' ')
            elif next1 == '/':
                if do_space:
                    write(' ')
                if next2 == '/':
                    # Line comment: treat it as a newline, but skip it
                    next2 = self.line_comment(next1, next2)
                    next1 = '\n'
                    next2, do_newline = self.newline(
                        previous_non_space, next2, do_newline)
                elif next2 == '*':
                    self.block_comment(next1, next2)
                    next2 = read(1)
                    if previous_non_space in space_strings:
                        do_space = True
                    next1 = previous
                else:
                    if previous_non_space in '{(,=:[?!&|;' or self.is_return:
                        self.regex_literal(next1, next2)
                        # hackish: after regex literal next1 is still /
                        # (it was the initial /, now it's the last /)
                        next2 = read(1)
                    else:
                        write('/')
            else:
                if do_newline:
                    write('\n')
                    do_newline = False
                    do_space = False
                if do_space:
                    do_space = False
                    write(' ')

                write(next1)
                if next1 in self.quote_chars:
                    in_quote = next1
                    quote_buf = []

            if next1 >= '!':
                previous_non_space = next1

            if next1 == '\\':
                escape_slash_count += 1
            else:
                escape_slash_count = 0

            previous = next1
            next1 = next2

    def regex_literal(self, next1, next2):
        assert next1 == '/'  # otherwise we should not be called!

        self.return_buf = ''

        read = self.ins.read
        write = self.outs.write

        in_char_class = False

        write('/')

        next = next2
        while next != '/' or in_char_class:
            write(next)
            if next == '\\':
                write(read(1))  # whatever is next is escaped
            elif next == '[':
                write(read(1))  # character class cannot be empty
                in_char_class = True
            elif next == ']':
                in_char_class = False
            next = read(1)

        write('/')

    def line_comment(self, next1, next2):
        assert next1 == next2 == '/'

        read = self.ins.read

        while next1 and next1 not in '\r\n':
            next1 = read(1)
        while next1 and next1 in '\r\n':
            next1 = read(1)

        return next1

    def block_comment(self, next1, next2):
        assert next1 == '/'
        assert next2 == '*'

        read = self.ins.read

        # Skip past first /* and avoid catching on /*/...*/
        next1 = read(1)
        next2 = read(1)
        while next1 != '*' or next2 != '/':
            next1 = next2
            next2 = read(1)

    def newline(self, previous_non_space, next2, do_newline):
        read = self.ins.read

        if previous_non_space and (
                        previous_non_space in self.newlineend_strings
                        or previous_non_space > '~'):
            while 1:
                if next2 < '!':
                    next2 = read(1)
                    if not next2:
                        break
                else:
                    if next2 in self.newlinestart_strings \
                            or next2 > '~' or next2 == '/':
                        do_newline = True
                    break

        return next2, do_newline

if __name__ == '__main__':
	import sys, os, glob

#for f in sys.argv[1:]:
#    with open(f, 'r') as js:
#        minifier = JavascriptMinify(js, sys.stdout)
#        minifier.minify()
#    sys.stdout.write('\n')

	minifier = JavascriptMinify(sys.stdin, sys.stdout)
	minifier.minify()
	sys.stdout.write('\n')




#!/usr/bin/python

import json
import sys

buf = sys.stdin.read()
#json.load method converts JSON string to Python Object
parsed = json.loads(buf)
sys.stdout.write(json.dumps(parsed, indent=2, sort_keys=True))



'''
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
from zencoding.parser import css, xml
import re

def is_stop_char(token):
	return token['type'] in '{};:'

def char_at(text, pos):
	"""
	Returns character at specified index of text.
	If index if out of range, returns empty string
	"""
	return text[pos] if pos < len(text) else ''

def calculate_nl_length(content, pos):
	"""
	Calculates newline width at specified position in content
	@param content: str
	@param pos: int
	@return: int
	"""
	if char_at(content, pos) == '\r' and char_at(content, pos + 1) == '\n':
		return 2
		
	return 1

def post_process_optimized(optimized, original):
	"""
	Post-process optimized tokens: collapse tokens for complex values
	@param optimized: Optimized tokens
	@type optimized: list
	@param original: Original preprocessed tokens 
	@type original: list 
	"""
	for token in optimized:
		child = None
		if token['type'] == 'value':
			token['children'] = []
			child = None
			
			subtoken_start = token['ref_start_ix']
				
			while subtoken_start <= token['ref_end_ix']:
				subtoken = original[subtoken_start]
				if subtoken['type'] != 'white':
					if not child:
						child = [subtoken['start'], subtoken['end']]
					else:
						child[1] = subtoken['end']
				elif child:
					token['children'].append(child)
					child = None
				
				subtoken_start += 1	
			
			if child: # push last token
				token['children'].append(child)
	
	return optimized

def make_token(type='', value='', pos=0, ix=0):
	value = value or ''
	return {
		'type': type or '',
		'content': value,
		'start': pos,
		'end': pos + len(value),
		# Reference token index that starts current token
		'ref_start_ix': ix,
		# Reference token index that ends current token
		'ref_end_ix': ix
	}

def parse_css(source, offset=0):
	"""
	Parses CSS and optimizes parsed chunks
	@param source: CSS source code fragment
	@type source: str
	@param offset: Offset of CSS fragment inside whole document
	@type offset: int
	@return: list
	"""
	return optimize_css(css.parse(source), offset, source)

def parse_html(tag, offset=0):
	"""
	Parses HTML and optimizes parsed chunks
	@param source: HTML source code fragment
	@type source: str
	@param offset: Offset of HTML fragment inside whole document
	@type offset: int
	@return: list
	"""
	tokens = xml.parse(tag)
	result = []
	i = 0
	loop = 1000 # infinite loop protection
		
	try:
		while loop:
			loop -= 1
			t = tokens['next']()
			if not t:
				break
			else:
				result.append(make_token(t['style'], t['content'], offset + i, 0))
				i += len(t['value'])
	except xml.StopIteration:
		pass
	
	return result

class ExtList(list):
	def __init__(self):
		super(ExtList, self).__init__()
		self.original = []
		

def optimize_css(tokens, offset, content):
	"""
	Optimizes parsed CSS tokens: combines selector chunks, complex values
	into a single chunk
	@param tokens: Tokens produced by <code>CSSEX.lex()</code>
	@type tokens: list
	@param offset: CSS rule offset in source code (character index)
	@type offset: int
	@param content: Original CSS source code
	@type content: str
	@return: list of optimized tokens  
	"""
	offset = offset or 0
	result = ExtList()
	_o = 0
	i = 0
	delta = 0
	in_rules = False
	in_value = False
	acc_tokens = {
		'selector': None,
		'value': None
	}
	orig_tokens = []
	acc_type = None
		
	def add_token(token, type):
		if type and type in acc_tokens:
			if not acc_tokens[type]:
				acc_tokens[type] = make_token(type, token['value'], offset + delta + token['charstart'], i)
				result.append(acc_tokens[type])
			else:
				acc_tokens[type]['content'] += token['value']
				acc_tokens[type]['end'] += len(token['value'])
				acc_tokens[type]['ref_end_ix'] = i
		else:
			result.append(make_token(token['type'], token['value'], offset + delta + token['charstart'], i))
		
	for i, token in enumerate(tokens):
		token = tokens[i]
		acc_type = None
		
		if token['type'] == 'line':
			delta += _o
			nl_size = content and calculate_nl_length(content, delta) or 1
			tok_value = nl_size == 1 and '\n' or '\r\n'
			
			orig_tokens.append(make_token(token['type'], tok_value, offset + delta))
			
			result.append(make_token(token['type'], tok_value, offset + delta, i))
			delta += nl_size
			_o = 0
			
			continue
		
		orig_tokens.append(make_token(token['type'], token['value'], offset + delta + token['charstart']))
		
		# use charstart and length because of incorrect charend 
		# computation for whitespace
		_o = token['charstart'] + len(token['value'])
		
		if token['type'] != 'white':
			if token['type'] == '{':
				in_rules = True
				acc_tokens['selector'] = None
			elif in_rules:
				if token['type'] == ':':
					in_value = True
				elif token['type'] == ';':
					in_value = False
					acc_tokens['value'] = None
				elif token['type'] == '}':
					in_value = in_rules = False
					acc_tokens['value'] = None
				elif in_value or acc_tokens['value']:
					acc_type = 'value'
			elif acc_tokens['selector'] or (not in_rules and not is_stop_char(token)):
				# start selector token
				acc_type = 'selector'
			
			add_token(token, acc_type)
		else:
			# whitespace token, decide where it should be
			if i < len(tokens) - 1 and is_stop_char(tokens[i + 1]):
				continue
			
			if acc_tokens['selector'] or acc_tokens['value']:
				add_token(token, acc_tokens['selector'] and 'selector' or 'value')
	
	result.original = orig_tokens
	return post_process_optimized(result, orig_tokens)

def extract_css_rule(content, pos, is_backward=False):
	"""
	 Extracts single CSS selector definition from source code
	 @param {String} content CSS source code
	 @type content: str
	 @param pos: Character position where to start source code extraction
	 @type pos: int
	"""
	result = '' 
	c_len = len(content)
	offset = pos 
	brace_pos = -1
	
	# search left until we find rule edge
	while offset >= 0:
		ch = content[offset]
		if ch == '{':
			brace_pos = offset
			break
		elif ch == '}' and not is_backward:
			offset += 1
			break
		
		offset -= 1
	
	# search right for full rule set
	while offset < c_len:
		ch = content[offset]
		if ch == '{':
			brace_pos = offset
		elif ch == '}':
			if brace_pos != -1:
				result = content[brace_pos:offset + 1]
			break
		
		offset += 1
	
	if result:
		# find CSS selector
		offset = brace_pos - 1
		selector = ''
		while offset >= 0:
			ch = content[offset]
			if ch in '{}/\\<>': break
			offset -= 1
		
		# also trim whitespace
		re_white = re.compile(r'^[\s\n\r]+', re.MULTILINE)
		selector = re.sub(re_white, '', content[offset + 1:brace_pos])
		return (brace_pos - len(selector), brace_pos + len(result))
	
	return None

# function alias
token = make_token


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Generic formatting filter: creates proper indentation for each tree node,
placing "%s" placeholder where the actual output should be. You can use
this filter to preformat tree and then replace %s placeholder to whatever you
need. This filter should't be called directly from editor as a part 
of abbreviation.
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
"""
import re
import zencoding.utils

child_token = '${child}'
placeholder = '%s'

def get_newline():
	return zencoding.utils.get_newline()


def get_indentation():
	return zencoding.utils.get_indentation()

def has_block_sibling(item):
	"""
	Test if passed node has block-level sibling element
	@type item: ZenNode
	@return: bool
	"""
	return item.parent and item.parent.has_block_children()

def is_very_first_child(item):
	"""
	Test if passed itrem is very first child of the whole tree
	@type tree: ZenNode
	"""
	return item.parent and not item.parent.parent and not item.previous_sibling

def should_break_line(node, profile):
	"""
	Need to add line break before element
	@type node: ZenNode
	@type profile: dict
	@return: bool
	"""
	if not profile['inline_break']:
		return False
		
	# find toppest non-inline sibling
	while node.previous_sibling and node.previous_sibling.is_inline():
		node = node.previous_sibling
	
	if not node.is_inline():
		return False
		
	# calculate how many inline siblings we have
	node_count = 1
	node = node.next_sibling
	while node:
		if node.is_inline():
			node_count += 1
		else:
			break
		node = node.next_sibling
	
	return node_count >= profile['inline_break']

def should_break_child(node, profile):
	"""
	 Need to add newline because <code>item</code> has too many inline children
	 @type node: ZenNode
	 @type profile: dict
	 @return: bool
	"""
	# we need to test only one child element, because 
	# has_block_children() method will do the rest
	return node.children and should_break_line(node.children[0], profile)

def process_snippet(item, profile, level=0):
	"""
	Processes element with <code>snippet</code> type
	@type item: ZenNode
	@type profile: dict
	@param level: Depth level
	@type level: int
	"""
	data = item.source.value;
		
	if not data:
		# snippet wasn't found, process it as tag
		return process_tag(item, profile, level)
		
	item.start = placeholder
	item.end = placeholder
	
	padding = item.parent.padding if item.parent else get_indentation() * level 
	
	if not is_very_first_child(item):
		item.start = get_newline() + padding + item.start
	
	# adjust item formatting according to last line of <code>start</code> property
	parts = data.split(child_token)
	lines = zencoding.utils.split_by_lines(parts[0] or '')
	padding_delta = get_indentation()
		
	if len(lines) > 1:
		m = re.match(r'^(\s+)', lines[-1])
		if m:
			padding_delta = m.group(1)
	
	item.padding = padding + padding_delta
	
	return item

def process_tag(item, profile, level=0):
	"""
	Processes element with <code>tag</code> type
	@type item: ZenNode
	@type profile: dict
	@param level: Depth level
	@type level: int
	"""
	if not item.name:
		# looks like it's a root element
		return item
	
	item.start = placeholder
	item.end = placeholder
	
	is_unary = item.is_unary() and not item.children
		
	# formatting output
	if profile['tag_nl'] is not False:
		padding = item.parent.padding if item.parent else get_indentation() * level
		force_nl = profile['tag_nl'] is True
		should_break = should_break_line(item, profile)
		
		# formatting block-level elements
		if ((item.is_block() or should_break) and item.parent) or force_nl:
			# snippet children should take different formatting
			if not item.parent or (item.parent.type != 'snippet' and not is_very_first_child(item)):
				item.start = get_newline() + padding + item.start
				
			if item.has_block_children() or should_break_child(item, profile) or (force_nl and not is_unary):
				item.end = get_newline() + padding + item.end
				
			if item.has_tags_in_content() or (force_nl and not item.has_children() and not is_unary):
				item.start += get_newline() + padding + get_indentation()
			
		elif item.is_inline() and has_block_sibling(item) and not is_very_first_child(item):
			item.start = get_newline() + padding + item.start
		elif item.is_inline() and item.has_block_children():
			item.end = get_newline() + padding + item.end
		
		item.padding = padding + get_indentation()
	
	return item

@zencoding.filter('_format')
def process(tree, profile, level=0):
	"""
	Processes simplified tree, making it suitable for output as HTML structure
	@type item: ZenNode
	@type profile: dict
	@param level: Depth level
	@type level: int
	"""
	
	for item in tree.children:
		if item.type == 'tag':
			item = process_tag(item, profile, level)
		else:
			item = process_snippet(item, profile, level)
		
		if item.content:
			item.content = zencoding.utils.pad_string(item.content, item.padding)
			
		process(item, profile, level + 1)
	
	return tree


#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Filter for trimming "select" attributes from some tags that contains
child elements
@author Sergey Chikuyonok (serge.che@gmail.com)
@link http://chikuyonok.ru
'''
import re
import zencoding

tags = {
	'xsl:variable': 1,
	'xsl:with-param': 1
}

re_attr = re.compile(r'\s+select\s*=\s*([\'"]).*?\1')

def trim_attribute(node):
	"""
	Removes "select" attribute from node
	@type node: ZenNode
	"""
	node.start = re_attr.sub('', node.start)

@zencoding.filter('xsl')
def process(tree, profile):
	for item in tree.children:
		if item.type == 'tag' and item.name.lower() in tags and item.children:
			trim_attribute(item)
		
		process(item, profile)